/* Copyright (c) 2016 AO MCST. All rights reserved.
 * Distributed under the terms of MIT License.
 */

#include "f2c.h"

static DB sinpi (DB);
const DB constlgamma[];
const DB constsin[], constsin_lo[];

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                        %
%  ПРОГРАММА  ВЫЧИСЛЯЕТ  НАТУРАЛЬНЫЙ  ЛОГАРИФМ  ГАММА-   %
%  ФУНКЦИИ . НЕДОПУСТИМЫМ  ЗНАЧЕНИЕМ  АРГУМЕНТА  ЯВЛЯЕТ- %
%  СЯ  X = -0,-1,-2,-3... ИЛИ Х > 2.55998332785164e+305. %
%  ЗНАК ГАММА-ФУНКЦИИ СОХРАНЯЕТСЯ В ПАРАМЕТРЕ signgamp.  %
%                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

DB __ieee754_lgamma_r (DB X, int *signgamp)

#define KLN2PION2_hi 9.189386367797851562e-01   /* log(2*pi)/2 */
#define KLN2PION2_lo -1.035751124144696674e-07
#define KLNPI_hi 1.144729614257812500e+00       /* log(pi) */
#define KLNPI_lo 2.715915876741434096e-07
#define K2 4.00006488775094945494e-01
#define K1 6.66666666629825839191e-01
#define KA5 -1.70289272020273209380e-03
#define KA4 8.38867008019779341353e-04
#define KA3 -5.95219917750874784541e-04
#define KA2 7.93650741475810108603e-04
#define KA1 -2.77777777772278018530e-03
#define KA0 8.33333333333333239101e-02
#define KMAX_LGAMMA 2.55998332785163832577e+305
{
    _type_double_bits arg, y0, y1, ln_hi;
    DB y, yd, ydv2, y_lo, ln_lo, *A0;
    LL i0;
    int exp, tmpsign;

    *signgamp = 1;
    y1.value = dfabs (X);
    exp = exponentabs (y1);
    if (exp == 0x7ff)
        return X * X;           /* X == NaN или Inf */
    arg.value = X;
    if (arg.llong < 0) {
        y0.value = sinpi (X);
        if (y0.llong & 0x8000000000000000LL)
            *signgamp = -1;
        if (y0.value == 0)
            return SVID_libm_err (X, X, 15); /* lgamma pole */
        if (exp < 0x3b9) /* |x|<2**-70 */
            return -__ieee754_log (y1.value);
/*    return log(PI/dfabs(X*y0.value))-lgamma(y1.value);*/
        return KLNPI_lo - __ieee754_log (dfabs (X * y0.value)) + KLNPI_hi - __ieee754_lgamma_r (y1.value, &tmpsign);
    }
    if (X > KMAX_LGAMMA)
        return SVID_libm_err (X, X, 14); /* lgamma overflow */
    if (exp < 0x3b9) /* |x|<2**-70 */
        return -__ieee754_log (X);
    exp -= 1023;                /* вычисление log(x) */
    y0.llong = (i0 = arg.llong & 0xfe00000000000LL) | 0x3ff0000000000000LL;
    y1.llong = (arg.llong & 0xfffffffffffffLL) | 0x3ff0000000000000LL;
    yd = (y1.value - y0.value) / (y1.value + y0.value);
    ydv2 = yd * yd;
    y0.llong = (LL)((unsigned *) constln)[i0 >>= 45] << 32;
    ln_lo = yd + yd + exp * KLN2_lo + constln_lo[i0] + K2 * yd * ydv2 * (ydv2 + K1 / K2);
    yd = y0.value + exp * KLN2_hi;
    ln_hi.value = ln_lo + yd;
    ln_hi.llong &= ~0xffffffffLL;
    ln_lo += yd - ln_hi.value;

    if (X < 9.0) {
        y1.value = (arg.llong < 0x3fde000000000000LL)      /* X < 0.46875 */
            ? X + 1.0 : X;
        y0.llong = (y1.llong + 0x2000000000000LL) & 0x7ffc000000000000LL;
        A0 = (DB *) (constlgamma + ((y0.llong - 0x3fe0000000000000LL) >> 50) * 13);
        y = y1.value - y0.value;
        yd = y * y;
        y0.value = y;
        y0.llong &= ~0xffffffffLL;
        y_lo = y - y0.value;
        y = ((A0[0] + A0[1] * y) * yd + yd * yd * (A0[2] + A0[3] * y + yd)) /
            (A0[4] + A0[5] * y + yd * (A0[6] + A0[7] * y + A0[8] * yd)) + (y * A0[10] + A0[12] + y_lo * A0[9]);
        yd = y0.value * A0[9];
        ydv2 = A0[11];
        if (y0.llong < 0x3f00000000000000LL)
            y += yd;
        else
            ydv2 += yd;
        return (arg.llong < 0x3fde000000000000LL)      /* X < 0.46875 */
            ? y - ln_lo + ydv2 - ln_hi.value : y + ydv2;
    }
    y = 1.0 / X;
    yd = y * y;
    ydv2 = y * yd;
    y = KA0 * y + KLN2PION2_lo + KA2 * ydv2 * (KA1 / KA2 + yd) +
        KA5 * ydv2 * (yd * yd) * (KA3 / KA5 + yd * (KA4 / KA5 + yd));
    arg.llong &= ~0xffffffffLL;
    y_lo = X - arg.value;
    yd = ln_hi.value - 1.0;
    return y + (y_lo * yd + (X - 0.5) * ln_lo) + (KLN2PION2_hi - 0.5 * ln_hi.value) + arg.value * yd;
}

strong_alias (__ieee754_lgamma_r, __lgamma_r_finite)
weak_alias (__ieee754_lgamma_r, __lgamma_r)
weak_alias (__ieee754_lgamma_r, lgamma_r)
#ifdef NO_LONG_DOUBLE
strong_alias (__ieee754_lgamma_r, __lgammal_r)
weak_alias (__ieee754_lgamma_r, lgammal_r)
#endif

#undef KA5
#undef KA3
#define KSI5 2.55016318262268599932e+00
#define KSI3 -5.16771278004966345110e+00
#define KCO4 4.05871053465977537045e+00
#define KCO2 -4.93480220054429980499e+00
#define KA9 8.21037050252602184882e-02
#define KA7 -5.99264444814193681515e-01
#define KA5 2.55016403980824032318e+00
#define KA3 -5.16771278004995025018e+00
#define KA1_hi 3.141592025756835938e+00
#define KA1_lo 6.27832957300056854749e-07

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                        %
%     ПРОГРАММА  ВЫЧИСЛЯЕТ  ЗНАЧЕНИЕ  ФУНКЦИИ  'СИНУС'.  %
%  АРГУМЕНТ  В  ДОЛЯХ ПИ,  Т.Е.  SINPI(X) = SIN(X*PI) .  %
%                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

static DB sinpi (DB X)
{
    _type_double_bits arg, rez;
    DB y, yv2, t;
    LL ind, k;

    y = arg.value = dfabs (X);
    rez.value = (X - X) * X;
    ind = arg.llong >> 52;
    if (ind >= 0x433)
        return rez.value;       /* |x| >= 2^52 */
    if (ind >= 0x42a) {         /* |x| >= 2^43 */
        k = (arg.llong << (ind - 0x42a)) & 1023;
        t = 0;
    }
    else {
        arg.value += DVAIN52 / 512.0;
        t = y - (arg.value - DVAIN52 / 512.0);
        k = arg.llong;
    }
    ind = k & 511;
    yv2 = t * t;
    if (ind >= 256) {
        ind = 512 - ind;
        t = -t;
    }
    if (ind <= 24) {
        t = (y + DVAIN52) - DVAIN52;
        k = (y == t) ? 0 : k + 24;
        y -= t;
        yv2 = y * y;
        arg.value = y;
        arg.llong &= ~0xffffffffLL;
        t = arg.value * KA1_hi;
        y = (y - arg.value) * PI + arg.value * KA1_lo + KA5 * y * yv2 *
            (KA3 / KA5 + yv2) + y * yv2 * (yv2 * yv2) * (KA9 * yv2 + KA7);
    }
    else {
        y = constsin[ind];
        t = KCO4 * yv2 * (yv2 + KCO2 / KCO4) * y + constsin_lo[ind] +
            constsin[256 - ind] * (KSI5 * t * yv2 * (yv2 + KSI3 / KSI5) + PI * t);
    }
    arg.value = y + t;
    arg.llong += rez.llong + ((k >> 9) << 63);
    return arg.value;
}

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%    КОНСТАНТЫ НАТУРАЛЬНОГО ЛОГАРИФМА  ГАММА-ФУНКЦИИ     %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

const DB constlgamma[] = {
    4.66021186742133657770e+01, /* P0  X0=0.5 */
    1.56521579466157539016e+02, /* P1  */
    1.56655854756398513298e+02, /* P2  */
    4.55213702057098303126e+01, /* P3  */
    1.88871272972479623697e+01, /* Q0  */
    8.49056051417395917259e+01, /* Q1  */
    1.28937948492081616405e+02, /* Q2  */
    7.45656971493358229198e+01, /* Q3  */
    1.25312639224433396548e+01, /* Q4  */
   -1.96350955963134765625e+00, /* A1_hi */
   -4.66390075819595555263e-07, /* A1_lo */
    5.72364807128906250000e-01, /* A0_hi */
    1.35795793837066059347e-07, /* A0_lo */
    9.68573136429995541857e+01, /* P0  X0=0.625 */
    2.64546829648199674195e+02, /* P1  */
    2.16652843054315244410e+02, /* P2  */
    5.20333326472879836361e+01, /* P3  */
    5.69550704653264547067e+01, /* Q0  */
    2.05065075787251917815e+02, /* Q1  */
    2.49297728760791381127e+02, /* Q2  */
    1.15338180775323267271e+02, /* Q3  */
    1.55385219735354953617e+01, /* Q4  */
   -1.45270919799804687500e+00, /* A1_hi */
    4.33421480300399041629e-07, /* A1_lo */
    3.60829591751098632813e-01, /* A0_hi */
   -9.62621584516383780875e-08, /* A0_lo */
    1.92275611307358234178e+02, /* P0  X0=0.75 */
    4.39764537928914490262e+02, /* P1  */
    3.01946465275477434166e+02, /* P2  */
    6.07754704505934940370e+01, /* P3  */
    1.51286164538502121297e+02, /* Q0  */
    4.51214946647779945912e+02, /* Q1  */
    4.53316403162338546431e+02, /* Q2  */
    1.72662699482970571887e+02, /* Q3  */
    1.90607553210935330042e+01, /* Q4  */
   -1.08586120605468750000e+00, /* A1_hi */
    3.26268215329305191823e-07, /* A1_lo */
    2.03280925750732421875e-01, /* A0_hi */
    2.56805629496051470718e-08, /* A0_lo */
    3.42448234847200677638e+02, /* P0  X0=0.875 */
    6.75116212717429237863e+02, /* P1  */
    4.00114131817692804606e+02, /* P2  */
    6.94904819955398844560e+01, /* P3  */
    3.41468057970444760648e+02, /* Q0  */
    8.69473739058135780052e+02, /* Q1  */
    7.44721664212436046469e+02, /* Q2  */
    2.41293218154162374193e+02, /* Q3  */
    2.25810871459622896726e+01, /* Q4  */
   -8.04017066955566406250e-01, /* A1_hi */
   -4.59212897628404490474e-09, /* A1_lo */
    8.58587026596069335938e-02, /* A0_hi */
    4.56572738990840122794e-09, /* A0_lo */
    6.11391362488191670885e+02, /* P0  X0=1.0 */
    1.04925146097046535296e+03, /* P1  */
    5.40175028610568300529e+02, /* P2  */
    8.10018213380482348371e+01, /* P3  */
    7.43362758192062025681e+02, /* Q0  */
    1.63788478391367576654e+03, /* Q1  */
    1.21015456643245396067e+03, /* Q2  */
    3.36640699893875747050e+02, /* Q3  */
    2.68383269829856996453e+01, /* Q4  */
   -5.77215671539306640625e-01, /* A1_hi */
    6.63777378046013286276e-09, /* A1_lo */
    0.00000000000000000000e+00, /* A0_hi */
    0.00000000000000000000e+00, /* A0_lo */
    1.30339788856155266825e+03, /* P0  X0=1.25 */
    1.82110074635308890918e+03, /* P1  */
    7.67610831649339095787e+02, /* P2  */
    9.50359228598315057646e+01, /* P3  */
    2.17717556388761910196e+03, /* Q0  */
    3.84670814295547852773e+03, /* Q1  */
    2.28252161640457625391e+03, /* Q2  */
    5.11421945235349508015e+02, /* Q3  */
    3.29738514179622255346e+01, /* Q4  */
   -2.27453470230102539063e-01, /* A1_hi */
   -6.31461628742003325540e-08, /* A1_lo */
   -9.82718467712402343750e-02, /* A0_hi */
    1.03494270728962946696e-08, /* A0_lo */
    2.64728395128486977776e+03, /* P0  X0=1.5 */
    3.08803140606273947520e+03, /* P1  */
    1.08707901337232058911e+03, /* P2  */
    1.12536195694580507944e+02, /* P3  */
    5.66383765408849516752e+03, /* Q0  */
    8.28066731266500597627e+03, /* Q1  */
    4.06155357081585932315e+03, /* Q2  */
    7.51761522655960391451e+02, /* Q3  */
    4.00061949019195916237e+01, /* Q4  */
    3.64899635314941406250e-02, /* A1_hi */
    1.04470823791500987247e-08, /* A1_lo */
   -1.20782256126403808594e-01, /* A0_hi */
    1.84911585862463823721e-08, /* A0_lo */
    4.87754607799522080959e+03, /* P0  X0=1.75 */
    4.87686513698887100231e+03, /* P1  */
    1.47063807472757681956e+03, /* P2  */
    1.30460144584034222843e+02, /* P3  */
    1.27667429440342856562e+04, /* Q0  */
    1.58943587042604016936e+04, /* Q1  */
    6.63100270405273259066e+03, /* Q2  */
    1.04310542412072148069e+03, /* Q3  */
    4.71445075776846960842e+01, /* Q4  */
    2.47472405433654785156e-01, /* A1_hi */
    4.81132063783926219052e-08, /* A1_lo */
   -8.44011306762695312500e-02, /* A0_hi */
    9.65578397529189846019e-09, /* A0_lo */
    9.05116168003804572617e+03, /* P0  X0=2.0 */
    7.83189761424499900840e+03, /* P1  */
    2.03385340676669299009e+03, /* P2  */
    1.54335304556349758944e+02, /* P3  */
    2.80684868277180738447e+04, /* Q0  */
    3.01499805606057153874e+04, /* Q1  */
    1.08130489710792914139e+04, /* Q2  */
    1.45568152166032186506e+03, /* Q3  */
    5.59566642013757282450e+01, /* Q4  */
    4.22784328460693359375e-01, /* A1_hi */
    6.63777378034985433408e-09, /* A1_lo */
    0.00000000000000000000e+00, /* A0_hi */
    0.00000000000000000000e+00, /* A0_lo */
    2.02715012926540406787e+04, /* P0  X0=2.5 */
    1.41960551304398709392e+04, /* P1  */
    2.99030384354294718579e+03, /* P2  */
    1.85112807607050271191e+02, /* P3  */
    8.26804554041164446962e+04, /* Q0  */
    7.11764611572501136314e+04, /* Q1  */
    2.04788115115147093804e+04, /* Q2  */
    2.21656060662029918540e+03, /* Q3  */
    6.87898457316799711889e+01, /* Q4  */
    7.03156471252441406250e-01, /* A1_hi */
    1.69392801777135153786e-07, /* A1_lo */
    2.84682750701904296875e-01, /* A0_hi */
    1.19771014862735908629e-07, /* A0_lo */
    4.21482622223681162113e+04, /* P0  X0=3.0 */
    2.46029670905109476353e+04, /* P1  */
    4.31394676096342976744e+03, /* P2  */
    2.22055283949190300032e+02, /* P3  */
    2.13444550675167435429e+05, /* Q0  */
    1.52356701061762789701e+05, /* Q1  */
    3.63075033505712040664e+04, /* Q2  */
    3.25098871089628945683e+03, /* Q3  */
    8.33607004366818173811e+01, /* Q4  */
    9.22784328460693359375e-01, /* A1_hi */
    6.63777377943768943661e-09, /* A1_lo */
    6.93147182464599609375e-01, /* A0_hi */
   -1.90465429996045016671e-09, /* A0_lo */
    7.81081322437143250449e+04, /* P0  X0=3.5 */
    3.90929641281837139962e+04, /* P1  */
    5.87300344158197586290e+03, /* P2  */
    2.58846833247604061991e+02, /* P3  */
    4.72869976874496602712e+05, /* Q0  */
    2.88297789180164003096e+05, /* Q1  */
    5.86451376513310997429e+04, /* Q2  */
    4.47911449641922295906e+03, /* Q3  */
    9.78884054807988392103e+01, /* Q4  */
    1.10315704345703125000e+00, /* A1_hi */
   -4.02811788062893149430e-07, /* A1_lo */
    1.20097351074218750000e+00, /* A0_hi */
    9.16048867248155556745e-08, /* A0_lo */
    1.45678423967305635908e+05, /* P0  X0=4.0 */
    6.30889835844802311910e+04, /* P1  */
    8.16335572999082060266e+03, /* P2  */
    3.07616302903683063396e+02, /* P3  */
    1.02654433704254010651e+06, /* Q0  */
    5.41062924461752318173e+05, /* Q1  */
    9.48626625653459285419e+04, /* Q2  */
    6.21813060925783537642e+03, /* Q3  */
    1.15900150372971575717e+02, /* Q4  */
    1.25611782073974609375e+00, /* A1_hi */
   -1.52307945619631500176e-07, /* A1_lo */
    1.79175949096679687500e+00, /* A0_hi */
   -2.17387418742056786246e-08, /* A0_lo */
    3.24569117230538929282e+05, /* P0  X0=5.0 */
    1.13689388160128272323e+05, /* P1  */
    1.19488685074296919190e+04, /* P2  */
    3.68373549085523366460e+02, /* P3  */
    2.93299098730687557723e+06, /* Q0  */
    1.24288383787446949324e+06, /* Q1  */
    1.75642684052851780935e+05, /* Q2  */
    9.31381946252077151785e+03, /* Q3  */
    1.41216401778917146068e+02, /* Q4  */
    1.50611782073974609375e+00, /* A1_hi */
   -1.52307945624151555151e-07, /* A1_lo */
    3.17805290222167968750e+00, /* A0_hi */
    9.28126265932112370905e-07, /* A0_lo */
    6.79451520638982153489e+05, /* P0  X0=6.0 */
    1.97828098068658571612e+05, /* P1  */
    1.72800600393461535834e+04, /* P2  */
    4.42691380218328117229e+02, /* P3  */
    7.49437949405657054729e+06, /* Q0  */
    2.63380382908696397288e+06, /* Q1  */
    3.08623405548721496821e+05, /* Q2  */
    1.35663742500229352246e+04, /* Q3  */
    1.70461132703826060767e+02, /* Q4  */
    1.70611763000488281250e+00, /* A1_hi */
    3.84269176597404883037e-08, /* A1_lo */
    4.78749084472656250000e+00, /* A0_hi */
    8.98055483494243275820e-07, /* A0_lo */
    1.27203419641365931053e+06, /* P0  X0=7.0 */
    3.16563772566899646017e+05, /* P1  */
    2.36335575223717599310e+04, /* P2  */
    5.17468757657413906087e+02, /* P3  */
    1.65688589289404389596e+07, /* Q0  */
    4.96977665522765844399e+06, /* Q1  */
    4.96978716350382709675e+05, /* Q2  */
    1.86411675790546558232e+04, /* Q3  */
    1.99832902837166847433e+02, /* Q4  */
    1.87278461456298828125e+00, /* A1_hi */
   -2.79464521141957048949e-07, /* A1_lo */
    6.57925033569335937500e+00, /* A0_hi */
    8.76316741620059400613e-07, /* A0_lo */
    2.40000445918985826282e+06, /* P0  X0=8.0 */
    5.15105261449440576339e+05, /* P1  */
    3.30278608526881823722e+04, /* P2  */
    6.16818734370333304974e+02, /* P3  */
    3.60531511797139780033e+07, /* Q0  */
    9.33563772953928582527e+06, /* Q1  */
    8.03804339302434442805e+05, /* Q2  */
    2.58608285929429528700e+04, /* Q3  */
    2.36439717826488849308e+02, /* Q4  */
    2.01564216613769531250e+00, /* A1_hi */
   -6.88182085314787925809e-07, /* A1_lo */
    8.52516174316406250000e+00, /* A0_hi */
   -3.82098648199865401768e-07, /* A0_lo */
};

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
/*         КОНСТАНТЫ  ФУНКЦИЙ  SIN  и  COS              */
/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

const DB constsin[] = {
    0.0000000000000000000e+00,     /* SIN (PI*  0/512) */
    6.1358846491544753596e-03,     /* SIN (PI*  1/512) */
    1.2271538285719926079e-02,     /* SIN (PI*  2/512) */
    1.8406729905804820927e-02,     /* SIN (PI*  3/512) */
    2.4541228522912288032e-02,     /* SIN (PI*  4/512) */
    3.0674803176636625934e-02,     /* SIN (PI*  5/512) */
    3.6807222941358832324e-02,     /* SIN (PI*  6/512) */
    4.2938256934940823077e-02,     /* SIN (PI*  7/512) */
    4.9067674327418014255e-02,     /* SIN (PI*  8/512) */
    5.5195244349689939809e-02,     /* SIN (PI*  9/512) */
    6.1320736302208577783e-02,     /* SIN (PI* 10/512) */
    6.7443919563664057898e-02,     /* SIN (PI* 11/512) */
    7.3564563599667423529e-02,     /* SIN (PI* 12/512) */
    7.9682437971430121147e-02,     /* SIN (PI* 13/512) */
    8.5797312344439890462e-02,     /* SIN (PI* 14/512) */
    9.1908956497132728625e-02,     /* SIN (PI* 15/512) */
    9.8017140329560601994e-02,     /* SIN (PI* 16/512) */
    1.0412163387205457912e-01,     /* SIN (PI* 17/512) */
    1.1022220729388305881e-01,     /* SIN (PI* 18/512) */
    1.1631863091190476725e-01,     /* SIN (PI* 19/512) */
    1.2241067519921619850e-01,     /* SIN (PI* 20/512) */
    1.2849811079379317262e-01,     /* SIN (PI* 21/512) */
    1.3458070850712618632e-01,     /* SIN (PI* 22/512) */
    1.4065823933284923071e-01,     /* SIN (PI* 23/512) */
    1.4673047445536175166e-01,     /* SIN (PI* 24/512) */
    1.5279718525844342772e-01,     /* SIN (PI* 25/512) */
    1.5885814333386144168e-01,     /* SIN (PI* 26/512) */
    1.6491312048996992142e-01,     /* SIN (PI* 27/512) */
    1.7096188876030122636e-01,     /* SIN (PI* 28/512) */
    1.7700422041214875620e-01,     /* SIN (PI* 29/512) */
    1.8303988795514095852e-01,     /* SIN (PI* 30/512) */
    1.8906866414980621275e-01,     /* SIN (PI* 31/512) */
    1.9509032201612826785e-01,     /* SIN (PI* 32/512) */
    2.0110463484209191156e-01,     /* SIN (PI* 33/512) */
    2.0711137619221854971e-01,     /* SIN (PI* 34/512) */
    2.1311031991609137397e-01,     /* SIN (PI* 35/512) */
    2.1910124015686979723e-01,     /* SIN (PI* 36/512) */
    2.2508391135979283599e-01,     /* SIN (PI* 37/512) */
    2.3105810828067111964e-01,     /* SIN (PI* 38/512) */
    2.3702360599436720687e-01,     /* SIN (PI* 39/512) */
    2.4298017990326388995e-01,     /* SIN (PI* 40/512) */
    2.4892760574572016811e-01,     /* SIN (PI* 41/512) */
    2.5486565960451457155e-01,     /* SIN (PI* 42/512) */
    2.6079411791527551828e-01,     /* SIN (PI* 43/512) */
    2.6671275747489838633e-01,     /* SIN (PI* 44/512) */
    2.7262135544994898449e-01,     /* SIN (PI* 45/512) */
    2.7851968938505310521e-01,     /* SIN (PI* 46/512) */
    2.8440753721127184362e-01,     /* SIN (PI* 47/512) */
    2.9028467725446236764e-01,     /* SIN (PI* 48/512) */
    2.9615088824362382412e-01,     /* SIN (PI* 49/512) */
    3.0200594931922806700e-01,     /* SIN (PI* 50/512) */
    3.0784964004153489368e-01,     /* SIN (PI* 51/512) */
    3.1368174039889147666e-01,     /* SIN (PI* 52/512) */
    3.1950203081601567790e-01,     /* SIN (PI* 53/512) */
    3.2531029216226293414e-01,     /* SIN (PI* 54/512) */
    3.3110630575987640174e-01,     /* SIN (PI* 55/512) */
    3.3688985339222005069e-01,     /* SIN (PI* 56/512) */
    3.4266071731199439759e-01,     /* SIN (PI* 57/512) */
    3.4841868024943456842e-01,     /* SIN (PI* 58/512) */
    3.5416352542049038236e-01,     /* SIN (PI* 59/512) */
    3.5989503653498814878e-01,     /* SIN (PI* 60/512) */
    3.6561299780477387001e-01,     /* SIN (PI* 61/512) */
    3.7131719395183754341e-01,     /* SIN (PI* 62/512) */
    3.7700741021641825673e-01,     /* SIN (PI* 63/512) */
    3.8268343236508977173e-01,     /* SIN (PI* 64/512) */
    3.8834504669882629162e-01,     /* SIN (PI* 65/512) */
    3.9399204006104810860e-01,     /* SIN (PI* 66/512) */
    3.9962419984564682854e-01,     /* SIN (PI* 67/512) */
    4.0524131400498987091e-01,     /* SIN (PI* 68/512) */
    4.1084317105790394218e-01,     /* SIN (PI* 69/512) */
    4.1642956009763718256e-01,     /* SIN (PI* 70/512) */
    4.2200027079979968594e-01,     /* SIN (PI* 71/512) */
    4.2755509343028209432e-01,     /* SIN (PI* 72/512) */
    4.3309381885315196848e-01,     /* SIN (PI* 73/512) */
    4.3861623853852763765e-01,     /* SIN (PI* 74/512) */
    4.4412214457042923164e-01,     /* SIN (PI* 75/512) */
    4.4961132965460660005e-01,     /* SIN (PI* 76/512) */
    4.5508358712634382354e-01,     /* SIN (PI* 77/512) */
    4.6053871095824002363e-01,     /* SIN (PI* 78/512) */
    4.6597649576796617790e-01,     /* SIN (PI* 79/512) */
    4.7139673682599764856e-01,     /* SIN (PI* 80/512) */
    4.7679923006332213334e-01,     /* SIN (PI* 81/512) */
    4.8218377207912274852e-01,     /* SIN (PI* 82/512) */
    4.8755016014843595464e-01,     /* SIN (PI* 83/512) */
    4.9289819222978403687e-01,     /* SIN (PI* 84/512) */
    4.9822766697278185241e-01,     /* SIN (PI* 85/512) */
    5.0353838372571755869e-01,     /* SIN (PI* 86/512) */
    5.0883014254310703693e-01,     /* SIN (PI* 87/512) */
    5.1410274419322172659e-01,     /* SIN (PI* 88/512) */
    5.1935599016558958736e-01,     /* SIN (PI* 89/512) */
    5.2458968267846890622e-01,     /* SIN (PI* 90/512) */
    5.2980362468629466822e-01,     /* SIN (PI* 91/512) */
    5.3499761988709721066e-01,     /* SIN (PI* 92/512) */
    5.4017147272989288130e-01,     /* SIN (PI* 93/512) */
    5.4532498842204642231e-01,     /* SIN (PI* 94/512) */
    5.5045797293660480298e-01,     /* SIN (PI* 95/512) */
    5.5557023301960222474e-01,     /* SIN (PI* 96/512) */
    5.6066157619733602384e-01,     /* SIN (PI* 97/512) */
    5.6573181078361319739e-01,     /* SIN (PI* 98/512) */
    5.7078074588696728023e-01,     /* SIN (PI* 99/512) */
    5.7580819141784530075e-01,     /* SIN (PI*100/512) */
    5.8081395809576454508e-01,     /* SIN (PI*101/512) */
    5.8579785745643886033e-01,     /* SIN (PI*102/512) */
    5.9075970185887422842e-01,     /* SIN (PI*103/512) */
    5.9569930449243334347e-01,     /* SIN (PI*104/512) */
    6.0061647938386892665e-01,     /* SIN (PI*105/512) */
    6.0551104140432551392e-01,     /* SIN (PI*106/512) */
    6.1038280627630945272e-01,     /* SIN (PI*107/512) */
    6.1523159058062684548e-01,     /* SIN (PI*108/512) */
    6.2005721176328917865e-01,     /* SIN (PI*109/512) */
    6.2485948814238637708e-01,     /* SIN (PI*110/512) */
    6.2963823891492702537e-01,     /* SIN (PI*111/512) */
    6.3439328416364549822e-01,     /* SIN (PI*112/512) */
    6.3912444486377574380e-01,     /* SIN (PI*113/512) */
    6.4383154288979146507e-01,     /* SIN (PI*114/512) */
    6.4851440102211244508e-01,     /* SIN (PI*115/512) */
    6.5317284295377676408e-01,     /* SIN (PI*116/512) */
    6.5780669329707865693e-01,     /* SIN (PI*117/512) */
    6.6241577759017176111e-01,     /* SIN (PI*118/512) */
    6.6699992230363750665e-01,     /* SIN (PI*119/512) */
    6.7155895484701840063e-01,     /* SIN (PI*120/512) */
    6.7609270357531596036e-01,     /* SIN (PI*121/512) */
    6.8060099779545305059e-01,     /* SIN (PI*122/512) */
    6.8508366777270038136e-01,     /* SIN (PI*123/512) */
    6.8954054473706692462e-01,     /* SIN (PI*124/512) */
    6.9397146088965400900e-01,     /* SIN (PI*125/512) */
    6.9837624940897285355e-01,     /* SIN (PI*126/512) */
    7.0275474445722530245e-01,     /* SIN (PI*127/512) */
    7.0710678118654752440e-01,     /* SIN (PI*128/512) */
    7.1143219574521644152e-01,     /* SIN (PI*129/512) */
    7.1573082528381865413e-01,     /* SIN (PI*130/512) */
    7.2000250796138162908e-01,     /* SIN (PI*131/512) */
    7.2424708295146692094e-01,     /* SIN (PI*132/512) */
    7.2846439044822519649e-01,     /* SIN (PI*133/512) */
    7.3265427167241283462e-01,     /* SIN (PI*134/512) */
    7.3681656887736987509e-01,     /* SIN (PI*135/512) */
    7.4095112535495909118e-01,     /* SIN (PI*136/512) */
    7.4505778544146596241e-01,     /* SIN (PI*137/512) */
    7.4913639452345932547e-01,     /* SIN (PI*138/512) */
    7.5318679904361248248e-01,     /* SIN (PI*139/512) */
    7.5720884650648454758e-01,     /* SIN (PI*140/512) */
    7.6120238548426181403e-01,     /* SIN (PI*141/512) */
    7.6516726562245892589e-01,     /* SIN (PI*142/512) */
    7.6910333764557963935e-01,     /* SIN (PI*143/512) */
    7.7301045336273696081e-01,     /* SIN (PI*144/512) */
    7.7688846567323245004e-01,     /* SIN (PI*145/512) */
    7.8073722857209447830e-01,     /* SIN (PI*146/512) */
    7.8455659715557523302e-01,     /* SIN (PI*147/512) */
    7.8834642762660626201e-01,     /* SIN (PI*148/512) */
    7.9210657730021235178e-01,     /* SIN (PI*149/512) */
    7.9583690460888353626e-01,     /* SIN (PI*150/512) */
    7.9953726910790503350e-01,     /* SIN (PI*151/512) */
    8.0320753148064490981e-01,     /* SIN (PI*152/512) */
    8.0684755354379927221e-01,     /* SIN (PI*153/512) */
    8.1045719825259479173e-01,     /* SIN (PI*154/512) */
    8.1403632970594836165e-01,     /* SIN (PI*155/512) */
    8.1758481315158369650e-01,     /* SIN (PI*156/512) */
    8.2110251499110467906e-01,     /* SIN (PI*157/512) */
    8.2458930278502526447e-01,     /* SIN (PI*158/512) */
    8.2804504525775575207e-01,     /* SIN (PI*159/512) */
    8.3146961230254523708e-01,     /* SIN (PI*160/512) */
    8.3486287498638005630e-01,     /* SIN (PI*161/512) */
    8.3822470555483804319e-01,     /* SIN (PI*162/512) */
    8.4155497743689840960e-01,     /* SIN (PI*163/512) */
    8.4485356524970707326e-01,     /* SIN (PI*164/512) */
    8.4812034480329725128e-01,     /* SIN (PI*165/512) */
    8.5135519310526514226e-01,     /* SIN (PI*166/512) */
    8.5455798836540052077e-01,     /* SIN (PI*167/512) */
    8.5772861000027206990e-01,     /* SIN (PI*168/512) */
    8.6086693863776727934e-01,     /* SIN (PI*169/512) */
    8.6397285612158673792e-01,     /* SIN (PI*170/512) */
    8.6704624551569265148e-01,     /* SIN (PI*171/512) */
    8.7008699110871141865e-01,     /* SIN (PI*172/512) */
    8.7309497841829009864e-01,     /* SIN (PI*173/512) */
    8.7607009419540660710e-01,     /* SIN (PI*174/512) */
    8.7901222642863347783e-01,     /* SIN (PI*175/512) */
    8.8192126434835502971e-01,     /* SIN (PI*176/512) */
    8.8479709843093778010e-01,     /* SIN (PI*177/512) */
    8.8763962040285394776e-01,     /* SIN (PI*178/512) */
    8.9044872324475788995e-01,     /* SIN (PI*179/512) */
    8.9322430119551532034e-01,     /* SIN (PI*180/512) */
    8.9596624975618515591e-01,     /* SIN (PI*181/512) */
    8.9867446569395384304e-01,     /* SIN (PI*182/512) */
    9.0134884704602201457e-01,     /* SIN (PI*183/512) */
    9.0398929312344333159e-01,     /* SIN (PI*184/512) */
    9.0659570451491536533e-01,     /* SIN (PI*185/512) */
    9.0916798309052237656e-01,     /* SIN (PI*186/512) */
    9.1170603200542985140e-01,     /* SIN (PI*187/512) */
    9.1420975570353065464e-01,     /* SIN (PI*188/512) */
    9.1667905992104266312e-01,     /* SIN (PI*189/512) */
    9.1911385169005774391e-01,     /* SIN (PI*190/512) */
    9.2151403934204194347e-01,     /* SIN (PI*191/512) */
    9.2387953251128675613e-01,     /* SIN (PI*192/512) */
    9.2621024213831134197e-01,     /* SIN (PI*193/512) */
    9.2850608047321556594e-01,     /* SIN (PI*194/512) */
    9.3076696107898373194e-01,     /* SIN (PI*195/512) */
    9.3299279883473888771e-01,     /* SIN (PI*196/512) */
    9.3518350993894757764e-01,     /* SIN (PI*197/512) */
    9.3733901191257492320e-01,     /* SIN (PI*198/512) */
    9.3945922360218991196e-01,     /* SIN (PI*199/512) */
    9.4154406518302077841e-01,     /* SIN (PI*200/512) */
    9.4359345816196036150e-01,     /* SIN (PI*201/512) */
    9.4560732538052132573e-01,     /* SIN (PI*202/512) */
    9.4758559101774113465e-01,     /* SIN (PI*203/512) */
    9.4952818059303666720e-01,     /* SIN (PI*204/512) */
    9.5143502096900836955e-01,     /* SIN (PI*205/512) */
    9.5330604035419383692e-01,     /* SIN (PI*206/512) */
    9.5514116830577072150e-01,     /* SIN (PI*207/512) */
    9.5694033573220886494e-01,     /* SIN (PI*208/512) */
    9.5870347489587155537e-01,     /* SIN (PI*209/512) */
    9.6043051941556581120e-01,     /* SIN (PI*210/512) */
    9.6212140426904159543e-01,     /* SIN (PI*211/512) */
    9.6377606579543986669e-01,     /* SIN (PI*212/512) */
    9.6539444169768937455e-01,     /* SIN (PI*213/512) */
    9.6697647104485210909e-01,     /* SIN (PI*214/512) */
    9.6852209427441731622e-01,     /* SIN (PI*215/512) */
    9.7003125319454399260e-01,     /* SIN (PI*216/512) */
    9.7150389098625177554e-01,     /* SIN (PI*217/512) */
    9.7293995220556014547e-01,     /* SIN (PI*218/512) */
    9.7433938278557586052e-01,     /* SIN (PI*219/512) */
    9.7570213003852854446e-01,     /* SIN (PI*220/512) */
    9.7702814265775435149e-01,     /* SIN (PI*221/512) */
    9.7831737071962763311e-01,     /* SIN (PI*222/512) */
    9.7956976568544053444e-01,     /* SIN (PI*223/512) */
    9.8078528040323044913e-01,     /* SIN (PI*224/512) */
    9.8196386910955526407e-01,     /* SIN (PI*225/512) */
    9.8310548743121632718e-01,     /* SIN (PI*226/512) */
    9.8421009238692907319e-01,     /* SIN (PI*227/512) */
    9.8527764238894124477e-01,     /* SIN (PI*228/512) */
    9.8630809724459864786e-01,     /* SIN (PI*229/512) */
    9.8730141815785838240e-01,     /* SIN (PI*230/512) */
    9.8825756773074949140e-01,     /* SIN (PI*231/512) */
    9.8917650996478097345e-01,     /* SIN (PI*232/512) */
    9.9005821026229710551e-01,     /* SIN (PI*233/512) */
    9.9090263542778002511e-01,     /* SIN (PI*234/512) */
    9.9170975366909952286e-01,     /* SIN (PI*235/512) */
    9.9247953459870999816e-01,     /* SIN (PI*236/512) */
    9.9321194923479453310e-01,     /* SIN (PI*237/512) */
    9.9390697000235604155e-01,     /* SIN (PI*238/512) */
    9.9456457073425545212e-01,     /* SIN (PI*239/512) */
    9.9518472667219688624e-01,     /* SIN (PI*240/512) */
    9.9576741446765979398e-01,     /* SIN (PI*241/512) */
    9.9631261218277801263e-01,     /* SIN (PI*242/512) */
    9.9682029929116571497e-01,     /* SIN (PI*243/512) */
    9.9729045667869021614e-01,     /* SIN (PI*244/512) */
    9.9772306664419160985e-01,     /* SIN (PI*245/512) */
    9.9811811290014920713e-01,     /* SIN (PI*246/512) */
    9.9847558057329475221e-01,     /* SIN (PI*247/512) */
    9.9879545620517239271e-01,     /* SIN (PI*248/512) */
    9.9907772775264538289e-01,     /* SIN (PI*249/512) */
    9.9932238458834950090e-01,     /* SIN (PI*250/512) */
    9.9952941750109316308e-01,     /* SIN (PI*251/512) */
    9.9969881869620422012e-01,     /* SIN (PI*252/512) */
    9.9983058179582342202e-01,     /* SIN (PI*253/512) */
    9.9992470183914454092e-01,     /* SIN (PI*254/512) */
    9.9998117528260114266e-01,     /* SIN (PI*255/512) */
    1.0000000000000000000e+00      /* SIN (PI*256/512) */
};

const DB constsin_lo[] = {
    0.0000000000000000000e+00,     /* мл. ч. SIN (PI*  0/512) */
    9.0545257482474932516e-20,     /* мл. ч. SIN (PI*  1/512) */
    6.9197907640283169421e-19,     /* мл. ч. SIN (PI*  2/512) */
    7.4195533812833164270e-19,     /* мл. ч. SIN (PI*  3/512) */
   -9.1868490125778778742e-20,     /* мл. ч. SIN (PI*  4/512) */
   -1.6936054844107921251e-20,     /* мл. ч. SIN (PI*  5/512) */
    6.1060088803529837706e-19,     /* мл. ч. SIN (PI*  6/512) */
    2.8351940588660908879e-18,     /* мл. ч. SIN (PI*  7/512) */
   -6.7961037205182805346e-19,     /* мл. ч. SIN (PI*  8/512) */
   -1.3340299860891102952e-18,     /* мл. ч. SIN (PI*  9/512) */
   -5.1181134064638111899e-19,     /* мл. ч. SIN (PI* 10/512) */
   -6.9221796556983634347e-18,     /* мл. ч. SIN (PI* 11/512) */
   -2.7784941506273594233e-18,     /* мл. ч. SIN (PI* 12/512) */
   -4.4867664311373037928e-18,     /* мл. ч. SIN (PI* 13/512) */
   -3.3881893830684030431e-18,     /* мл. ч. SIN (PI* 14/512) */
    4.7631594854274565672e-18,     /* мл. ч. SIN (PI* 15/512) */
   -1.6345823622442559994e-18,     /* мл. ч. SIN (PI* 16/512) */
    6.5760254085385101961e-18,     /* мл. ч. SIN (PI* 17/512) */
   -5.6789503537823231340e-19,     /* мл. ч. SIN (PI* 18/512) */
    1.0294914877509703836e-18,     /* мл. ч. SIN (PI* 19/512) */
    2.8354501489965337145e-18,     /* мл. ч. SIN (PI* 20/512) */
    3.8198603954988800543e-18,     /* мл. ч. SIN (PI* 21/512) */
   -9.1670359171480698051e-18,     /* мл. ч. SIN (PI* 22/512) */
   -7.9193932965524735294e-18,     /* мл. ч. SIN (PI* 23/512) */
    3.7269471470465681328e-18,     /* мл. ч. SIN (PI* 24/512) */
   -7.6313573938416831988e-18,     /* мл. ч. SIN (PI* 25/512) */
   -4.0163200573859079309e-18,     /* мл. ч. SIN (PI* 26/512) */
   -7.0405288319166735923e-19,     /* мл. ч. SIN (PI* 27/512) */
    9.1919980181759087701e-18,     /* мл. ч. SIN (PI* 28/512) */
    6.7329300635408170033e-18,     /* мл. ч. SIN (PI* 29/512) */
    7.7349918688637378882e-18,     /* мл. ч. SIN (PI* 30/512) */
   -7.6208955803527782164e-18,     /* мл. ч. SIN (PI* 31/512) */
   -7.9910790684617306616e-18,     /* мл. ч. SIN (PI* 32/512) */
    1.1010032669300738262e-17,     /* мл. ч. SIN (PI* 33/512) */
   -1.0613362528971355520e-17,     /* мл. ч. SIN (PI* 34/512) */
    1.2031873821063859439e-17,     /* мл. ч. SIN (PI* 35/512) */
   -3.6513812299150778228e-19,     /* мл. ч. SIN (PI* 36/512) */
    3.9507040822556510365e-18,     /* мл. ч. SIN (PI* 37/512) */
    1.0129787149761867937e-17,     /* мл. ч. SIN (PI* 38/512) */
    8.8544852285039919203e-18,     /* мл. ч. SIN (PI* 39/512) */
   -8.7514315297196638066e-18,     /* мл. ч. SIN (PI* 40/512) */
   -8.1783436100020984214e-18,     /* мл. ч. SIN (PI* 41/512) */
   -1.3602299806901461305e-19,     /* мл. ч. SIN (PI* 42/512) */
    4.2721420983550075890e-18,     /* мл. ч. SIN (PI* 43/512) */
    2.0941222578826687261e-17,     /* мл. ч. SIN (PI* 44/512) */
    7.8697166076387850434e-18,     /* мл. ч. SIN (PI* 45/512) */
   -1.0030273719543543237e-17,     /* мл. ч. SIN (PI* 46/512) */
    2.2209268510661474364e-17,     /* мл. ч. SIN (PI* 47/512) */
   -1.8927978707774250306e-17,     /* мл. ч. SIN (PI* 48/512) */
   -2.0220736360876938091e-17,     /* мл. ч. SIN (PI* 49/512) */
   -1.7167666235262473995e-17,     /* мл. ч. SIN (PI* 50/512) */
    2.7074088527245186294e-17,     /* мл. ч. SIN (PI* 51/512) */
    1.4560447299968912621e-17,     /* мл. ч. SIN (PI* 52/512) */
   -1.3981562491096625554e-17,     /* мл. ч. SIN (PI* 53/512) */
    7.9171249463765891523e-18,     /* мл. ч. SIN (PI* 54/512) */
   -2.7469465474778695314e-17,     /* мл. ч. SIN (PI* 55/512) */
   -4.2000940033475092353e-19,     /* мл. ч. SIN (PI* 56/512) */
    1.9261518449306317959e-17,     /* мл. ч. SIN (PI* 57/512) */
    3.6974420514204921517e-18,     /* мл. ч. SIN (PI* 58/512) */
   -1.6951763305764858848e-17,     /* мл. ч. SIN (PI* 59/512) */
   -1.7601687123839281199e-17,     /* мл. ч. SIN (PI* 60/512) */
    1.6217898770457547173e-17,     /* мл. ч. SIN (PI* 61/512) */
    3.4749239648238265572e-19,     /* мл. ч. SIN (PI* 62/512) */
   -2.7255302041956929448e-18,     /* мл. ч. SIN (PI* 63/512) */
   -1.0050772696461587804e-17,     /* мл. ч. SIN (PI* 64/512) */
   -1.0053770598398716153e-17,     /* мл. ч. SIN (PI* 65/512) */
    9.7649241641239334098e-18,     /* мл. ч. SIN (PI* 66/512) */
   -1.5065497476189372513e-17,     /* мл. ч. SIN (PI* 67/512) */
    9.9111401942899881754e-18,     /* мл. ч. SIN (PI* 68/512) */
   -2.4219835190355500468e-17,     /* мл. ч. SIN (PI* 69/512) */
   -2.5475580413131732672e-17,     /* мл. ч. SIN (PI* 70/512) */
    4.3543266994128530868e-18,     /* мл. ч. SIN (PI* 71/512) */
    9.4111898162954724244e-18,     /* мл. ч. SIN (PI* 72/512) */
    1.1224283329847516861e-17,     /* мл. ч. SIN (PI* 73/512) */
   -2.0883315831075090366e-17,     /* мл. ч. SIN (PI* 74/512) */
   -2.4218874422178314439e-17,     /* мл. ч. SIN (PI* 75/512) */
    4.8831924232035245907e-18,     /* мл. ч. SIN (PI* 76/512) */
   -1.2379906758116471521e-17,     /* мл. ч. SIN (PI* 77/512) */
    1.8488777492177870674e-17,     /* мл. ч. SIN (PI* 78/512) */
   -3.3023547778235134906e-18,     /* мл. ч. SIN (PI* 79/512) */
    6.5166781360690132534e-18,     /* мл. ч. SIN (PI* 80/512) */
   -1.0293515338305794794e-17,     /* мл. ч. SIN (PI* 81/512) */
   -2.5861500925520441957e-17,     /* мл. ч. SIN (PI* 82/512) */
    1.4231090931273652995e-17,     /* мл. ч. SIN (PI* 83/512) */
   -1.0257831676562184575e-18,     /* мл. ч. SIN (PI* 84/512) */
   -1.6126383830540799249e-17,     /* мл. ч. SIN (PI* 85/512) */
   -1.6731308204967497856e-17,     /* мл. ч. SIN (PI* 86/512) */
    4.7836968268014128672e-17,     /* мл. ч. SIN (PI* 87/512) */
   -4.5712707523615622410e-17,     /* мл. ч. SIN (PI* 88/512) */
   -5.5331248144171145110e-17,     /* мл. ч. SIN (PI* 89/512) */
   -4.3068869040082345270e-17,     /* мл. ч. SIN (PI* 90/512) */
   -4.8067841706482341348e-17,     /* мл. ч. SIN (PI* 91/512) */
   -5.3683132708358135432e-17,     /* мл. ч. SIN (PI* 92/512) */
    2.7072447965935839960e-17,     /* мл. ч. SIN (PI* 93/512) */
   -4.1517817538384259328e-17,     /* мл. ч. SIN (PI* 94/512) */
   -8.3319903015807665106e-18,     /* мл. ч. SIN (PI* 95/512) */
    4.7094109405616766192e-17,     /* мл. ч. SIN (PI* 96/512) */
   -7.3956418153476147851e-18,     /* мл. ч. SIN (PI* 97/512) */
   -3.4096079596590468211e-17,     /* мл. ч. SIN (PI* 98/512) */
    2.4568151455566206249e-17,     /* мл. ч. SIN (PI* 99/512) */
   -3.7909495458942736830e-17,     /* мл. ч. SIN (PI*100/512) */
    1.8585338586613408331e-17,     /* мл. ч. SIN (PI*101/512) */
   -3.7485501964311289746e-18,     /* мл. ч. SIN (PI*102/512) */
   -4.7013584170659539603e-17,     /* мл. ч. SIN (PI*103/512) */
   -1.3438641936579467237e-17,     /* мл. ч. SIN (PI*104/512) */
   -4.6398198229461933852e-17,     /* мл. ч. SIN (PI*105/512) */
   -3.1202672493305675738e-17,     /* мл. ч. SIN (PI*106/512) */
   -2.2563265648229167397e-17,     /* мл. ч. SIN (PI*107/512) */
    2.6231417767266948808e-17,     /* мл. ч. SIN (PI*108/512) */
   -2.7983410837681117181e-17,     /* мл. ч. SIN (PI*109/512) */
    3.3671846037243897494e-17,     /* мл. ч. SIN (PI*110/512) */
    4.1115334049626803791e-17,     /* мл. ч. SIN (PI*111/512) */
    1.0420901929280034962e-17,     /* мл. ч. SIN (PI*112/512) */
    1.2416796312271042107e-17,     /* мл. ч. SIN (PI*113/512) */
   -3.2084798795046887061e-17,     /* мл. ч. SIN (PI*114/512) */
    3.9870270313386832914e-18,     /* мл. ч. SIN (PI*115/512) */
    8.5695642060026231262e-18,     /* мл. ч. SIN (PI*116/512) */
    1.9580943058468544261e-17,     /* мл. ч. SIN (PI*117/512) */
   -2.2615508885764590853e-17,     /* мл. ч. SIN (PI*118/512) */
    3.5284364997428164718e-17,     /* мл. ч. SIN (PI*119/512) */
   -4.0489037749296692169e-17,     /* мл. ч. SIN (PI*120/512) */
    3.7256902248049467807e-17,     /* мл. ч. SIN (PI*121/512) */
    2.8473293354522047252e-17,     /* мл. ч. SIN (PI*122/512) */
    2.5948135194645136349e-17,     /* мл. ч. SIN (PI*123/512) */
   -1.5889323294806789031e-17,     /* мл. ч. SIN (PI*124/512) */
    7.4345076956280144720e-18,     /* мл. ч. SIN (PI*125/512) */
    4.8988282435667769572e-17,     /* мл. ч. SIN (PI*126/512) */
    2.5278294383629821683e-18,     /* мл. ч. SIN (PI*127/512) */
   -4.8336466567264565137e-17,     /* мл. ч. SIN (PI*128/512) */
    7.9643298613856828036e-18,     /* мл. ч. SIN (PI*129/512) */
   -5.1581018476410261174e-17,     /* мл. ч. SIN (PI*130/512) */
   -2.5689658854462333917e-17,     /* мл. ч. SIN (PI*131/512) */
    2.9198471334403006666e-17,     /* мл. ч. SIN (PI*132/512) */
    1.1924642323370722388e-19,     /* мл. ч. SIN (PI*133/512) */
    1.8918673481573518447e-17,     /* мл. ч. SIN (PI*134/512) */
   -2.8932468101656294533e-17,     /* мл. ч. SIN (PI*135/512) */
   -1.4708616952297344605e-17,     /* мл. ч. SIN (PI*136/512) */
    1.5078686911877864676e-17,     /* мл. ч. SIN (PI*137/512) */
   -4.4729078447011889925e-17,     /* мл. ч. SIN (PI*138/512) */
   -3.7937789838736539325e-17,     /* мл. ч. SIN (PI*139/512) */
   -1.9909098777335500607e-17,     /* мл. ч. SIN (PI*140/512) */
    3.5315510881690552806e-17,     /* мл. ч. SIN (PI*141/512) */
   -3.2707225612534596744e-17,     /* мл. ч. SIN (PI*142/512) */
    5.1546455184564819544e-17,     /* мл. ч. SIN (PI*143/512) */
   -3.2565907033649770891e-17,     /* мл. ч. SIN (PI*144/512) */
    7.7418602570672857777e-18,     /* мл. ч. SIN (PI*145/512) */
   -9.9198782066678804988e-18,     /* мл. ч. SIN (PI*146/512) */
   -8.5627965423173476673e-18,     /* мл. ч. SIN (PI*147/512) */
    3.4396993154059709720e-17,     /* мл. ч. SIN (PI*148/512) */
   -3.7087850202326467700e-17,     /* мл. ч. SIN (PI*149/512) */
   -3.0062724851910722582e-17,     /* мл. ч. SIN (PI*150/512) */
    2.0356723822005430718e-17,     /* мл. ч. SIN (PI*151/512) */
   -3.3060609804814909807e-17,     /* мл. ч. SIN (PI*152/512) */
    4.9220603766095549811e-17,     /* мл. ч. SIN (PI*153/512) */
    2.3520367349840499375e-17,     /* мл. ч. SIN (PI*154/512) */
   -5.2127351877042624980e-17,     /* мл. ч. SIN (PI*155/512) */
   -1.4883149812426771359e-17,     /* мл. ч. SIN (PI*156/512) */
    3.0715131609697681022e-17,     /* мл. ч. SIN (PI*157/512) */
   -2.6512360488868274105e-17,     /* мл. ч. SIN (PI*158/512) */
   -4.4196593225871528618e-17,     /* мл. ч. SIN (PI*159/512) */
    1.4073856984728029671e-18,     /* мл. ч. SIN (PI*160/512) */
    4.6048430609159657192e-17,     /* мл. ч. SIN (PI*161/512) */
   -3.5560085052855024673e-17,     /* мл. ч. SIN (PI*162/512) */
   -3.4095465391321557428e-17,     /* мл. ч. SIN (PI*163/512) */
   -4.3631360296879639050e-17,     /* мл. ч. SIN (PI*164/512) */
    1.8762563415239982452e-17,     /* мл. ч. SIN (PI*165/512) */
   -5.3279874446016206495e-17,     /* мл. ч. SIN (PI*166/512) */
   -1.2991124236396091441e-17,     /* мл. ч. SIN (PI*167/512) */
   -4.8183447936336621107e-17,     /* мл. ч. SIN (PI*168/512) */
   -3.0050048898573656844e-17,     /* мл. ч. SIN (PI*169/512) */
    4.1486355957361609722e-17,     /* мл. ч. SIN (PI*170/512) */
    3.0267859550930570552e-18,     /* мл. ч. SIN (PI*171/512) */
   -4.1888510868549966884e-17,     /* мл. ч. SIN (PI*172/512) */
    7.8424672990129905459e-18,     /* мл. ч. SIN (PI*173/512) */
    5.8729024235147688322e-18,     /* мл. ч. SIN (PI*174/512) */
   -4.7356837291118008341e-17,     /* мл. ч. SIN (PI*175/512) */
   -1.9843248405890561759e-17,     /* мл. ч. SIN (PI*176/512) */
   -9.4331469628972684174e-18,     /* мл. ч. SIN (PI*177/512) */
    1.2805091918587961743e-17,     /* мл. ч. SIN (PI*178/512) */
    1.1781931459051804283e-17,     /* мл. ч. SIN (PI*179/512) */
   -4.1161239151908899211e-18,     /* мл. ч. SIN (PI*180/512) */
    4.9025099114811812153e-17,     /* мл. ч. SIN (PI*181/512) */
    2.6316906461033014443e-17,     /* мл. ч. SIN (PI*182/512) */
   -1.3524789367698680409e-17,     /* мл. ч. SIN (PI*183/512) */
   -6.6097544687484300791e-18,     /* мл. ч. SIN (PI*184/512) */
    3.0506718955442022484e-17,     /* мл. ч. SIN (PI*185/512) */
   -3.6878564091359886770e-18,     /* мл. ч. SIN (PI*186/512) */
   -2.6913273175034129940e-17,     /* мл. ч. SIN (PI*187/512) */
   -3.6316182527814424747e-17,     /* мл. ч. SIN (PI*188/512) */
   -4.1733978698287566235e-17,     /* мл. ч. SIN (PI*189/512) */
   -2.6496484622344717249e-17,     /* мл. ч. SIN (PI*190/512) */
    4.1639843390684644731e-17,     /* мл. ч. SIN (PI*191/512) */
    1.7645047084336676578e-17,     /* мл. ч. SIN (PI*192/512) */
   -3.7303754586099052043e-17,     /* мл. ч. SIN (PI*193/512) */
   -2.3306639848485943613e-17,     /* мл. ч. SIN (PI*194/512) */
    1.9703775102838330183e-17,     /* мл. ч. SIN (PI*195/512) */
    4.2041415555384356535e-17,     /* мл. ч. SIN (PI*196/512) */
   -3.2609395302485065453e-17,     /* мл. ч. SIN (PI*197/512) */
   -3.6570926284362777130e-17,     /* мл. ч. SIN (PI*198/512) */
   -7.0152867943098654781e-18,     /* мл. ч. SIN (PI*199/512) */
   -2.7896379547698338761e-17,     /* мл. ч. SIN (PI*200/512) */
   -2.4093127844404213198e-17,     /* мл. ч. SIN (PI*201/512) */
    4.6019102478523738146e-17,     /* мл. ч. SIN (PI*202/512) */
    4.3417993852125991216e-17,     /* мл. ч. SIN (PI*203/512) */
   -7.5544151928043292623e-18,     /* мл. ч. SIN (PI*204/512) */
    3.1350584123266693709e-17,     /* мл. ч. SIN (PI*205/512) */
   -2.5190738779919933926e-17,     /* мл. ч. SIN (PI*206/512) */
    5.0088652955014666560e-17,     /* мл. ч. SIN (PI*207/512) */
    4.0553869861875699973e-17,     /* мл. ч. SIN (PI*208/512) */
   -4.3685014392372049994e-17,     /* мл. ч. SIN (PI*209/512) */
    2.4653904815317184828e-17,     /* мл. ч. SIN (PI*210/512) */
    1.5236770453846304466e-17,     /* мл. ч. SIN (PI*211/512) */
    2.6463950561220029646e-17,     /* мл. ч. SIN (PI*212/512) */
   -2.3745389918392154924e-17,     /* мл. ч. SIN (PI*213/512) */
    3.8496228837337864364e-17,     /* мл. ч. SIN (PI*214/512) */
    4.9475074918244774403e-17,     /* мл. ч. SIN (PI*215/512) */
    1.8365300348428843428e-17,     /* мл. ч. SIN (PI*216/512) */
   -7.9865421122289046752e-18,     /* мл. ч. SIN (PI*217/512) */
   -3.1311211122281802268e-17,     /* мл. ч. SIN (PI*218/512) */
    2.3041221476151514979e-18,     /* мл. ч. SIN (PI*219/512) */
   -2.5572556081259686515e-17,     /* мл. ч. SIN (PI*220/512) */
   -4.3353875751555995962e-17,     /* мл. ч. SIN (PI*221/512) */
   -2.1623082233344894050e-17,     /* мл. ч. SIN (PI*222/512) */
    1.5573991584990418900e-17,     /* мл. ч. SIN (PI*223/512) */
    1.8546939997825004667e-17,     /* мл. ч. SIN (PI*224/512) */
    2.1108443711513082677e-17,     /* мл. ч. SIN (PI*225/512) */
    4.2170007522888629085e-17,     /* мл. ч. SIN (PI*226/512) */
    4.7983922627050691340e-17,     /* мл. ч. SIN (PI*227/512) */
    2.3155637027900206214e-17,     /* мл. ч. SIN (PI*228/512) */
   -2.1520877103013339368e-17,     /* мл. ч. SIN (PI*229/512) */
   -5.2332261255715654773e-17,     /* мл. ч. SIN (PI*230/512) */
    2.7030607784372632194e-17,     /* мл. ч. SIN (PI*231/512) */
   -4.0987309937047111093e-17,     /* мл. ч. SIN (PI*232/512) */
   -1.7055485906233960539e-17,     /* мл. ч. SIN (PI*233/512) */
    1.5394565094566703655e-17,     /* мл. ч. SIN (PI*234/512) */
   -2.3372891311883651543e-18,     /* мл. ч. SIN (PI*235/512) */
    3.1093055095428905286e-17,     /* мл. ч. SIN (PI*236/512) */
    3.3096906261272264513e-17,     /* мл. ч. SIN (PI*237/512) */
   -1.8964849471123744545e-17,     /* мл. ч. SIN (PI*238/512) */
    3.6745069641528056011e-17,     /* мл. ч. SIN (PI*239/512) */
   -4.2486913678304407092e-17,     /* мл. ч. SIN (PI*240/512) */
   -2.3151908323094357787e-17,     /* мл. ч. SIN (PI*241/512) */
    1.1336497891624736379e-17,     /* мл. ч. SIN (PI*242/512) */
    4.7062820708615658895e-17,     /* мл. ч. SIN (PI*243/512) */
    9.1647695371101750944e-18,     /* мл. ч. SIN (PI*244/512) */
   -2.6394475274898719953e-17,     /* мл. ч. SIN (PI*245/512) */
    2.7935487558113835782e-17,     /* мл. ч. SIN (PI*246/512) */
   -2.2002931182778795483e-17,     /* мл. ч. SIN (PI*247/512) */
   -1.2291693337075464032e-17,     /* мл. ч. SIN (PI*248/512) */
    2.1419007653587031663e-17,     /* мл. ч. SIN (PI*249/512) */
   -4.2858538440845681328e-17,     /* мл. ч. SIN (PI*250/512) */
    2.0517917823755591050e-17,     /* мл. ч. SIN (PI*251/512) */
   -2.9851486403799754648e-17,     /* мл. ч. SIN (PI*252/512) */
    1.8825140517551118943e-17,     /* мл. ч. SIN (PI*253/512) */
    3.7931082512668012229e-17,     /* мл. ч. SIN (PI*254/512) */
    3.3568103522895584565e-17,     /* мл. ч. SIN (PI*255/512) */
    0.0000000000000000000e+00      /* мл. ч. SIN (PI*256/512) */
};
