/* Copyright (c) 2016-2018 AO MCST. All rights reserved.
 * Distributed under the terms of MIT License.
 */

#include "f2c.h"

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                        %
%     ПРОГРАММА  ВЫЧИСЛЯЕТ  ГЛАВНОЕ  ЗНАЧЕНИЕ  ФУНКЦИИ   %
%  'АРКТАНГЕНС'.  РЕЗУЛЬТАТ  В  РАДИАНАХ .               %
%                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#define KA9 1.00523913835726148843e-01
#define KA7 -1.42452689179072812273e-01
#define KA5 1.99994988952939798604e-01
#define KA3 -3.33333323562239806408e-01
#define KB3 -3.33327229950751257532e-01
#define KP3 5.14863724199257118914e-01
#define KP1 9.99999997171171308685e-01
#define KQ4 8.27683548536093944530e-02
#define KQ2 8.48196443819667386665e-01
#define LKA9 0x3fb9bbef6a63ef02LL
#define LKA7 0xbfc23be3c4a02cebLL
#define LKA5 0x3fc9996f90755851LL
#define LKA3 0xbfd555554ad779c1LL
#define LKB3 0xbfd5553bbbe04dccLL
#define LKP3 0x3fe079c37d2aa470LL
#define LKP1 0x3feffffffe7b3570LL
#define LKQ4 0x3fb5304e913d6f04LL
#define LKQ2 0x3feb246cde593f44LL

FL __atanf (FL X)
{
    _type_float_bits arg, arg1;
    DB y, yv2, yp;
    int ind;
#if __iset__ >= 6
    _type_float_bits ires;
    _type_double_bits ly, lyv2, lyv4, lyv5, lres, lres1, lres2;
#endif /* __iset__ < 6 */

    if (X == 0) return X; /* for X = -0.0 only */

    arg1.value = X;
    arg.value = ffabs (X);
    y = (DB) X;
    yv2 = y * y;

    if (arg.int0 <= 0x3e800000) { /* |x|<=0.25 */
#if __iset__ >= 6
        ly.value = y;
        lyv2.value = yv2;
        lyv4.value = yv2 * yv2;
        lyv5.value = lyv4.value * y;
        lres.value = KA3 * y;
        lres2.llong = __builtin_e2k_fmad (lres.llong, lyv2.llong, ly.llong);
        lres1.llong = __builtin_e2k_fmad (LKA9, lyv2.llong, LKA7);
        lres1.llong = __builtin_e2k_fmad (lres1.llong, lyv2.llong, LKA5);
        lres.llong = __builtin_e2k_fmad (lres1.llong, lyv5.llong, lres2.llong);
        return (FL) lres.value;
#else /* __iset__ < 6 */
        return (FL) (y + KA5 * y * yv2 * (KA3/KA5 + yv2) + yv2 * (yv2 * yv2) * (KA7 * y + KA9 * y * yv2));
#endif /* __iset__ < 6 */
    }
    if (arg.int0 <= 0x40800000) { /* |x|<=4 */
        ind = (arg.int0 - 0x3e800000) >> 17;
        arg.int0 = arg1.int0 & 0xfffe0000;
#if __iset__ >= 6
        ires.int0 = __builtin_e2k_fmas (arg1.int0, arg.int0, 0x3f800000);
        y = (DB) ((X - arg.value) / ires.value);
#else /* __iset__ < 6 */
        y = (DB) ((X - arg.value) / (1.0F + X * arg.value));
#endif /* __iset__ < 6 */
        yp = constnarctg[ind];
        yp = (arg1.int0 < 0) ? y - yp : y + yp;
#if __iset__ >= 6
        ly.value = yp;
        lres.value = y * y;
        lres1.value = KB3 * y;
        lres.llong = __builtin_e2k_fmad (lres1.llong, lres.llong, ly.llong);
        return (FL) lres.value;
#else /* __iset__ < 6 */
        return (FL) (yp + KB3 * y * (y * y));
#endif /* __iset__ < 6 */
    }
    if (arg.int0 <= 0x44800000) { /* |x|<=2^10 */
#if __iset__ >= 6
        lyv2.value = yv2;
        lres.llong = __builtin_e2k_fmad (LKP1, lyv2.llong, LKP3);
        lres1.value = yv2 + KQ2;
        lres1.llong = __builtin_e2k_fmad (lres1.llong, lyv2.llong, LKQ4);
        y = lres.value * y / lres1.value;
#else /* __iset__ < 6 */
        y = (KP3 + KP1 * yv2) * y / (KQ4 + yv2 * (KQ2 + yv2));
#endif /* __iset__ < 6 */
    }
    else y = (DB) (1.0F / X);
    yp = (arg1.int0 < 0) ? -PION2 : PION2;
    return (FL) (yp - y);
}

weak_alias (__atanf, atanf)

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   КОНСТАНТЫ  ДЛЯ  ФУНКЦИЙ   ATANF  И  АТAN2F.          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/
const DB constnarctg[]={
 2.4497866312686414347e-01,     /* ATAN (0x3e800000) */
 2.4865174119051325352e-01,     /* ATAN (0x3e820000) */
 2.5231798088642720668e-01,     /* ATAN (0x3e840000) */
 2.5597730301300553046e-01,     /* ATAN (0x3e860000) */
 2.5962962940825751179e-01,     /* ATAN (0x3e880000) */
 2.6327488295528239659e-01,     /* ATAN (0x3e8a0000) */
 2.6691298758740045205e-01,     /* ATAN (0x3e8c0000) */
 2.7054386829293658456e-01,     /* ATAN (0x3e8e0000) */
 2.7416745111965878934e-01,     /* ATAN (0x3e900000) */
 2.7778366317887326353e-01,     /* ATAN (0x3e920000) */
 2.8139243264917845888e-01,     /* ATAN (0x3e940000) */
 2.8499368877988129345e-01,     /* ATAN (0x3e960000) */
 2.8858736189407740991e-01,     /* ATAN (0x3e980000) */
 2.9217338339139875547e-01,     /* ATAN (0x3e9a0000) */
 2.9575168575043153663e-01,     /* ATAN (0x3e9c0000) */
 2.9932220253080743522e-01,     /* ATAN (0x3e9e0000) */
 3.0288486837497141657e-01,     /* ATAN (0x3ea00000) */
 3.0643961900963012646e-01,     /* ATAN (0x3ea20000) */
 3.0998639124688343038e-01,     /* ATAN (0x3ea40000) */
 3.1352512298504392474e-01,     /* ATAN (0x3ea60000) */
 3.1705575320914702875e-01,     /* ATAN (0x3ea80000) */
 3.2057822199115698636e-01,     /* ATAN (0x3eaa0000) */
 3.2409247048987172013e-01,     /* ATAN (0x3eac0000) */
 3.2759844095053086699e-01,     /* ATAN (0x3eae0000) */
 3.3109607670413210290e-01,     /* ATAN (0x3eb00000) */
 3.3458532216645897606e-01,     /* ATAN (0x3eb20000) */
 3.3806612283682546671e-01,     /* ATAN (0x3eb40000) */
 3.4153842529654171445e-01,     /* ATAN (0x3eb60000) */
 3.4500217720710513181e-01,     /* ATAN (0x3eb80000) */
 3.4845732730812206679e-01,     /* ATAN (0x3eba0000) */
 3.5190382541496478819e-01,     /* ATAN (0x3ebc0000) */
 3.5534162241616834566e-01,     /* ATAN (0x3ebe0000) */
 3.5877067027057224502e-01,     /* ATAN (0x3ec00000) */
 3.6219092200421221239e-01,     /* ATAN (0x3ec20000) */
 3.6560233170696687655e-01,     /* ATAN (0x3ec40000) */
 3.6900485452896442107e-01,     /* ATAN (0x3ec60000) */
 3.7239844667675420231e-01,     /* ATAN (0x3ec80000) */
 3.7578306540924893975e-01,     /* ATAN (0x3eca0000) */
 3.7915866903344180860e-01,     /* ATAN (0x3ecc0000) */
 3.8252521689990515164e-01,     /* ATAN (0x3ece0000) */
 3.8588266939807375211e-01,     /* ATAN (0x3ed00000) */
 3.8923098795132071714e-01,     /* ATAN (0x3ed20000) */
 3.9257013501182858040e-01,     /* ATAN (0x3ed40000) */
 3.9590007405526289608e-01,     /* ATAN (0x3ed60000) */
 3.9922076957525254315e-01,     /* ATAN (0x3ed80000) */
 4.0253218707768251283e-01,     /* ATAN (0x3eda0000) */
 4.0583429307480411996e-01,     /* ATAN (0x3edc0000) */
 4.0912705507916830028e-01,     /* ATAN (0x3ede0000) */
 4.1241044159738732278e-01,     /* ATAN (0x3ee00000) */
 4.1568442212372941347e-01,     /* ATAN (0x3ee20000) */
 4.1894896713355284090e-01,     /* ATAN (0x3ee40000) */
 4.2220404807658362678e-01,     /* ATAN (0x3ee60000) */
 4.2544963737004226623e-01,     /* ATAN (0x3ee80000) */
 4.2868570839162578601e-01,     /* ATAN (0x3eea0000) */
 4.3191223547234819380e-01,     /* ATAN (0x3eec0000) */
 4.3512919388924686803e-01,     /* ATAN (0x3eee0000) */
 4.3833655985795783039e-01,     /* ATAN (0x3ef00000) */
 4.4153431052516672883e-01,     /* ATAN (0x3ef20000) */
 4.4472242396093936145e-01,     /* ATAN (0x3ef40000) */
 4.4790087915093729221e-01,     /* ATAN (0x3ef60000) */
 4.5106965598852349908e-01,     /* ATAN (0x3ef80000) */
 4.5422873526676249556e-01,     /* ATAN (0x3efa0000) */
 4.5737809867032080957e-01,     /* ATAN (0x3efc0000) */
 4.6051772876727109507e-01,     /* ATAN (0x3efe0000) */
 4.6364760900080609352e-01,     /* ATAN (0x3f000000) */
 4.6987805797568693578e-01,     /* ATAN (0x3f020000) */
 4.7606933032276121942e-01,     /* ATAN (0x3f040000) */
 4.8222132422785374262e-01,     /* ATAN (0x3f060000) */
 4.8833395105640553524e-01,     /* ATAN (0x3f080000) */
 4.9440713507127537207e-01,     /* ATAN (0x3f0a0000) */
 5.0044081314729416121e-01,     /* ATAN (0x3f0c0000) */
 5.0643493448309673255e-01,     /* ATAN (0x3f0e0000) */
 5.1238946031073773213e-01,     /* ATAN (0x3f100000) */
 5.1830436360357801107e-01,     /* ATAN (0x3f120000) */
 5.2417962878291324280e-01,     /* ATAN (0x3f140000) */
 5.3001525142379313227e-01,     /* ATAN (0x3f160000) */
 5.3581123796046370433e-01,     /* ATAN (0x3f180000) */
 5.4156760539184500836e-01,     /* ATAN (0x3f1a0000) */
 5.4728438098743692475e-01,     /* ATAN (0x3f1c0000) */
 5.5296160199402832891e-01,     /* ATAN (0x3f1e0000) */
 5.5859931534356244143e-01,     /* ATAN (0x3f200000) */
 5.6419757736249764868e-01,     /* ATAN (0x3f220000) */
 5.6975645348297843107e-01,     /* ATAN (0x3f240000) */
 5.7527601795611782443e-01,     /* ATAN (0x3f260000) */
 5.8075635356767041362e-01,     /* ATAN (0x3f280000) */
 5.8619755135636064658e-01,     /* ATAN (0x3f2a0000) */
 5.9159971033511138394e-01,     /* ATAN (0x3f2c0000) */
 5.9696293721540150123e-01,     /* ATAN (0x3f2e0000) */
 6.0228734613496415218e-01,     /* ATAN (0x3f300000) */
 6.0757305838902242456e-01,     /* ATAN (0x3f320000) */
 6.1282020216524135670e-01,     /* ATAN (0x3f340000) */
 6.1802891228256184863e-01,     /* ATAN (0x3f360000) */
 6.2319932993406590427e-01,     /* ATAN (0x3f380000) */
 6.2833160243400965062e-01,     /* ATAN (0x3f3a0000) */
 6.3342588296914459356e-01,     /* ATAN (0x3f3c0000) */
 6.3848233035443757721e-01,     /* ATAN (0x3f3e0000) */
 6.4350110879328437097e-01,     /* ATAN (0x3f400000) */
 6.4848238764230059505e-01,     /* ATAN (0x3f420000) */
 6.5342634118076192706e-01,     /* ATAN (0x3f440000) */
 6.5833314838475598396e-01,     /* ATAN (0x3f460000) */
 6.6320299270609328612e-01,     /* ATAN (0x3f480000) */
 6.6803606185602015799e-01,     /* ATAN (0x3f4a0000) */
 6.7283254759376320830e-01,     /* ATAN (0x3f4c0000) */
 6.7759264551992515191e-01,     /* ATAN (0x3f4e0000) */
 6.8231655487474807131e-01,     /* ATAN (0x3f500000) */
 6.8700447834124500623e-01,     /* ATAN (0x3f520000) */
 6.9165662185319987110e-01,     /* ATAN (0x3f540000) */
 6.9627319440802359907e-01,     /* ATAN (0x3f560000) */
 7.0085440788445019233e-01,     /* ATAN (0x3f580000) */
 7.0540047686504903091e-01,     /* ATAN (0x3f5a0000) */
 7.0991161846352490716e-01,     /* ATAN (0x3f5c0000) */
 7.1438805215676903781e-01,     /* ATAN (0x3f5e0000) */
 7.1882999962162452690e-01,     /* ATAN (0x3f600000) */
 7.2323768457631787410e-01,     /* ATAN (0x3f620000) */
 7.2761133262651067621e-01,     /* ATAN (0x3f640000) */
 7.3195117111591656567e-01,     /* ATAN (0x3f660000) */
 7.3625742898142809700e-01,     /* ATAN (0x3f680000) */
 7.4053033661269274113e-01,     /* ATAN (0x3f6a0000) */
 7.4477012571607514868e-01,     /* ATAN (0x3f6c0000) */
 7.4897702918294140062e-01,     /* ATAN (0x3f6e0000) */
 7.5315128096219441378e-01,     /* ATAN (0x3f700000) */
 7.5729311593699244476e-01,     /* ATAN (0x3f720000) */
 7.6140276980557841657e-01,     /* ATAN (0x3f740000) */
 7.6548047896614446195e-01,     /* ATAN (0x3f760000) */
 7.6952648040565829746e-01,     /* ATAN (0x3f780000) */
 7.7354101159257349085e-01,     /* ATAN (0x3f7a0000) */
 7.7752431037334779340e-01,     /* ATAN (0x3f7c0000) */
 7.8147661487268826885e-01,     /* ATAN (0x3f7e0000) */
 7.8539816339744827900e-01,     /* ATAN (0x3f800000) */
 7.9314994610965516841e-01,     /* ATAN (0x3f820000) */
 8.0078156517804344183e-01,     /* ATAN (0x3f840000) */
 8.0829492461883234444e-01,     /* ATAN (0x3f860000) */
 8.1569192331622342174e-01,     /* ATAN (0x3f880000) */
 8.2297445274017166916e-01,     /* ATAN (0x3f8a0000) */
 8.3014439485206392710e-01,     /* ATAN (0x3f8c0000) */
 8.3720362018844940444e-01,     /* ATAN (0x3f8e0000) */
 8.4415398611317105093e-01,     /* ATAN (0x3f900000) */
 8.5099733522848708400e-01,     /* ATAN (0x3f920000) */
 8.5773549393604331392e-01,     /* ATAN (0x3f940000) */
 8.6437027113884046248e-01,     /* ATAN (0x3f960000) */
 8.7090345707565297584e-01,     /* ATAN (0x3f980000) */
 8.7733682227966947043e-01,     /* ATAN (0x3f9a0000) */
 8.8367211665345746230e-01,     /* ATAN (0x3f9c0000) */
 8.8991106865268165826e-01,     /* ATAN (0x3f9e0000) */
 8.9605538457134392694e-01,     /* ATAN (0x3fa00000) */
 9.0210674792164591285e-01,     /* ATAN (0x3fa20000) */
 9.0806681890190499296e-01,     /* ATAN (0x3fa40000) */
 9.1393723394628445522e-01,     /* ATAN (0x3fa60000) */
 9.1971960535041685780e-01,     /* ATAN (0x3fa80000) */
 9.2541552096731505284e-01,     /* ATAN (0x3faa0000) */
 9.3102654396827100314e-01,     /* ATAN (0x3fac0000) */
 9.3655421266373528599e-01,     /* ATAN (0x3fae0000) */
 9.4200004037946361013e-01,     /* ATAN (0x3fb00000) */
 9.4736551538348379164e-01,     /* ATAN (0x3fb20000) */
 9.5265210085970941645e-01,     /* ATAN (0x3fb40000) */
 9.5786123492427843740e-01,     /* ATAN (0x3fb60000) */
 9.6299433068093620580e-01,     /* ATAN (0x3fb80000) */
 9.6805277631201913646e-01,     /* ATAN (0x3fba0000) */
 9.7303793520181458554e-01,     /* ATAN (0x3fbc0000) */
 9.7795114608928257471e-01,     /* ATAN (0x3fbe0000) */
 9.8279372324732905408e-01,     /* ATAN (0x3fc00000) */
 9.8756695668600513738e-01,     /* ATAN (0x3fc20000) */
 9.9227211237719137316e-01,     /* ATAN (0x3fc40000) */
 9.9691043249849298213e-01,     /* ATAN (0x3fc60000) */
 1.0014831356942346385e+00,     /* ATAN (0x3fc80000) */
 1.0059914173515940305e+00,     /* ATAN (0x3fca0000) */
 1.0104364498900535452e+00,     /* ATAN (0x3fcc0000) */
 1.0148193830624914202e+00,     /* ATAN (0x3fce0000) */
 1.0191413442663497246e+00,     /* ATAN (0x3fd00000) */
 1.0234034388634409840e+00,     /* ATAN (0x3fd20000) */
 1.0276067505070687336e+00,     /* ATAN (0x3fd40000) */
 1.0317523414752334254e+00,     /* ATAN (0x3fd60000) */
 1.0358412530088001446e+00,     /* ATAN (0x3fd80000) */
 1.0398745056535936282e+00,     /* ATAN (0x3fda0000) */
 1.0438530996054669053e+00,     /* ATAN (0x3fdc0000) */
 1.0477780150574778073e+00,     /* ATAN (0x3fde0000) */
 1.0516502125483737640e+00,     /* ATAN (0x3fe00000) */
 1.0554706333116605776e+00,     /* ATAN (0x3fe20000) */
 1.0592401996245910389e+00,     /* ATAN (0x3fe40000) */
 1.0629598151564723096e+00,     /* ATAN (0x3fe60000) */
 1.0666303653157436226e+00,     /* ATAN (0x3fe80000) */
 1.0702527175953329142e+00,     /* ATAN (0x3fea0000) */
 1.0738277219158434139e+00,     /* ATAN (0x3fec0000) */
 1.0773562109661667385e+00,     /* ATAN (0x3fee0000) */
 1.0808390005411683266e+00,     /* ATAN (0x3ff00000) */
 1.0842768898761148133e+00,     /* ATAN (0x3ff20000) */
 1.0876706619775629026e+00,     /* ATAN (0x3ff40000) */
 1.0910210839504541624e+00,     /* ATAN (0x3ff60000) */
 1.0943289073211899254e+00,     /* ATAN (0x3ff80000) */
 1.0975948683564933361e+00,     /* ATAN (0x3ffa0000) */
 1.1008196883778886832e+00,     /* ATAN (0x3ffc0000) */
 1.1040040740716487999e+00,     /* ATAN (0x3ffe0000) */
 1.1071487177940904090e+00,     /* ATAN (0x40000000) */
 1.1133214788988714528e+00,     /* ATAN (0x40020000) */
 1.1193432352423422493e+00,     /* ATAN (0x40040000) */
 1.1252190398203147925e+00,     /* ATAN (0x40060000) */
 1.1309537439791603752e+00,     /* ATAN (0x40080000) */
 1.1365520059980955825e+00,     /* ATAN (0x400a0000) */
 1.1420182993488803280e+00,     /* ATAN (0x400c0000) */
 1.1473569206351774685e+00,     /* ATAN (0x400e0000) */
 1.1525719972156676096e+00,     /* ATAN (0x40100000) */
 1.1576674945164953101e+00,     /* ATAN (0x40120000) */
 1.1626472230398072760e+00,     /* ATAN (0x40140000) */
 1.1675148450760606256e+00,     /* ATAN (0x40160000) */
 1.1722738811284763027e+00,     /* ATAN (0x40180000) */
 1.1769277160585351893e+00,     /* ATAN (0x401a0000) */
 1.1814796049617557472e+00,     /* ATAN (0x401c0000) */
 1.1859326787832342731e+00,     /* ATAN (0x401e0000) */
 1.1902899496825316561e+00,     /* ATAN (0x40200000) */
 1.1945543161575231661e+00,     /* ATAN (0x40220000) */
 1.1987285679367760682e+00,     /* ATAN (0x40240000) */
 1.2028153906499123860e+00,     /* ATAN (0x40260000) */
 1.2068173702852524887e+00,     /* ATAN (0x40280000) */
 1.2107369974438444427e+00,     /* ATAN (0x402a0000) */
 1.2145766713987464769e+00,     /* ATAN (0x402c0000) */
 1.2183387039681923270e+00,     /* ATAN (0x402e0000) */
 1.2220253232109896668e+00,     /* ATAN (0x40300000) */
 1.2256386769522282787e+00,     /* ATAN (0x40320000) */
 1.2291808361470895061e+00,     /* ATAN (0x40340000) */
 1.2326537980902492198e+00,     /* ATAN (0x40360000) */
 1.2360594894780818631e+00,     /* ATAN (0x40380000) */
 1.2393997693305771612e+00,     /* ATAN (0x403a0000) */
 1.2426764317795913062e+00,     /* ATAN (0x403c0000) */
 1.2458912087297775440e+00,     /* ATAN (0x403e0000) */
 1.2490457723982544280e+00,     /* ATAN (0x40400000) */
 1.2521417377388039949e+00,     /* ATAN (0x40420000) */
 1.2551806647561303265e+00,     /* ATAN (0x40440000) */
 1.2581640607154498390e+00,     /* ATAN (0x40460000) */
 1.2610933822524403869e+00,     /* ATAN (0x40480000) */
 1.2639700373883393514e+00,     /* ATAN (0x404a0000) */
 1.2667953874547519533e+00,     /* ATAN (0x404c0000) */
 1.2695707489325107620e+00,     /* ATAN (0x404e0000) */
 1.2722973952087173188e+00,     /* ATAN (0x40500000) */
 1.2749765582558991728e+00,     /* ATAN (0x40520000) */
 1.2776094302370144540e+00,     /* ATAN (0x40540000) */
 1.2801971650398613622e+00,     /* ATAN (0x40560000) */
 1.2827408797442707566e+00,     /* ATAN (0x40580000) */
 1.2852416560252879485e+00,     /* ATAN (0x405a0000) */
 1.2877005414954010298e+00,     /* ATAN (0x405c0000) */
 1.2901185509887085345e+00,     /* ATAN (0x405e0000) */
 1.2924966677897853362e+00,     /* ATAN (0x40600000) */
 1.2948358448098624685e+00,     /* ATAN (0x40620000) */
 1.2971370057128062125e+00,     /* ATAN (0x40640000) */
 1.2994010459932607837e+00,     /* ATAN (0x40660000) */
 1.3016288340091961562e+00,     /* ATAN (0x40680000) */
 1.3038212119709955417e+00,     /* ATAN (0x406a0000) */
 1.3059789968891077905e+00,     /* ATAN (0x406c0000) */
 1.3081029814821876212e+00,     /* ATAN (0x406e0000) */
 1.3101939350475555468e+00,     /* ATAN (0x40700000) */
 1.3122526042957145531e+00,     /* ATAN (0x40720000) */
 1.3142797141505730973e+00,     /* ATAN (0x40740000) */
 1.3162759685169478363e+00,     /* ATAN (0x40760000) */
 1.3182420510168371131e+00,     /* ATAN (0x40780000) */
 1.3201786256958834009e+00,     /* ATAN (0x407a0000) */
 1.3220863377013738482e+00,     /* ATAN (0x407c0000) */
 1.3239658139330623410e+00,     /* ATAN (0x407e0000) */
 1.3258176636680325533e+00};    /* ATAN (0x40800000) */
