/* Copyright (c) 2016 AO MCST. All rights reserved.
 * Distributed under the terms of MIT License.
 */

#include <errno.h>
#include "f2c.h"

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                        %
%     ПРОГРАММА  ВЫЧИСЛЯЕТ  ЗНАЧЕНИЕ  ФУНКЦИИ  'СИНУС'.  %
%  АРГУМЕНТ  В  РАДИАНАХ.                                %
%                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

#undef KC4
#undef KC2
#undef KS5
#undef KS3
#define RS      1.38350580552821637120e+19  /* 3*2^62 = 0x43e8000000000000 */
#define PIO2_1  1.57079632679233327508e+00  /* первые 36 бит pi/2 = 0x3ff921fb54440000 */
#define PIO2_2  2.56334415158395578912e-12  /* вторые 36 бит pi/2 = 0x3d868c234c4c0000 */
#define PIO2_2T 1.05629990669874271122e-23L /* оставшиеся 64 бит pi/2 = 3fb2 0xcc51701b839a2520 */

//(sinl(sqrtl(y))-sqrtl(y))/y/sqrtl(y); b = DPION4 * DPION4; a = 0.0L; o=6; MAX=0.85506 or 0.22589
#define KS7 -7.5866971177069181028716e-13
#define KS6  1.6058531618986146839027e-10
#define KS5 -2.5052106232447577328029e-08
#define KS4  2.7557319219339166517547e-06
#define KS3 -1.9841269841265064946528e-04L
#define KS2  8.3333333333333314920845e-03L
#define KS1 -1.6666666666666666665508e-01L

//(cosl(sqrtl(y)) - 1.0L)/y; b = DPION4 * DPION4; a = 0.0L; o=7; MAX=0.0036623 or 8.093
#define KC8  4.7410866985075003361486e-14
#define KC7 -1.1470361263661413773148e-11
#define KC6  2.0876754983065436482112e-09
#define KC5 -2.7557319218191050044576e-07
#define KC4  2.4801587301578204650565e-05L
#define KC3 -1.3888888888888881875461e-03L
#define KC2  4.1666666666666666646067e-02L

LD __sinl (LD X)
{
    _type_long_double_bits arg;
    LD xl, xv2, xv4, fn0, a0, b0, w0;
    DB y0, y1;
    int k;

    arg.value = ldfabs (X);

    if (arg.value <= (PION2 / 2)) {     /* |x| <= pi/4 */

        if (arg.value == 0) { /* |x| = 0 */
            return X;
        }
        xv2 = X * X;
        xv4 = xv2 * xv2;
        return KS7 * X * xv4 * xv4 * (KS4 / KS7 + KS5 / KS7 * xv2 + xv4 * (KS6 / KS7 + xv2)) +
            ((xv2 * KS3 + KS2) * xv2 + KS1) * (xv2 * X) + X;
    }
    if (arg.value > 8.43314856532626152039e8) { /* |x| > 2^28 * pi = 0x41c921fb54442d18 */
        y0 = __rem_pio2l (X, &y1, &k);
        X = y0 + (LD) y1;
        xl = y0 - X + y1;
    }
    else {     /* |x| > pi/4, но |x| <= 2^28 * pi */
//        arg.value = X * K2ONPI + RS;
//        k = arg.twolong.long1;
//        fn0 = arg.value - RS;
        arg.value = X * K2ONPI + ((X < 0) ? -0.5 : 0.5);
        k = (int) arg.value;
        fn0 = (LD) k;
        a0 = X - fn0 * PIO2_1;
        w0 = fn0 * PIO2_2;
        b0 = a0 - w0;
        xl = (a0 - b0) - w0 - fn0 * PIO2_2T;
        X = b0 + xl;
        xl = b0 - X + xl;
    }
    xv2 = X * X;
    xv4 = xv2 * xv2;
    if (k & 1) { /* cos */
        arg.value = KC8 * xv2 * xv4 * xv4 * (KC5 / KC8 + KC6 / KC8 * xv2 + xv4 * (KC7 / KC8 + xv2)) +
            X * xl + ((KC4 * xv2 + KC3) * xv2 + KC2) * xv4 - 0.5 * xv2 + 1;
    }
    else { /* sin */
        arg.value = KS7 * X * xv4 * xv4 * (KS4 / KS7 + KS5 / KS7 * xv2 + xv4 * (KS6 / KS7 + xv2)) +
            xl + ((xv2 * KS3 + KS2) * xv2 + KS1) * (xv2 * X) + X;
    }
    if (k & 2) arg.value = -arg.value;
    return arg.value;
}

weak_alias (__sinl, sinl)

/*%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                        %
%  ПРОГРАММА  ПРИВЕДЕНИЯ  АРГУМЕНТА ТРИГОНОМЕТРИЧЕСКИХ   %
%  ФУНКЦИЙ  К  ДИАПАЗОНУ  ОТ  -PI/4  ДО  PI/4 .          %
%                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%*/

/* **************************************************************************
 *
 * Common argument reduction routine for circular trigonometric functions.
 *
 * Inputs:
 *  x = argument value to be reduced to [-Pi/4..Pi/4] interval;
 * Outputs:
 *  nseg = 0,1,2,3 = (k % 4).
 *  hi+lo "fraction" part of reduced value; the hi+lo pair returned is
 *        closer to "normalized", so |lo| < C*0.5*|hi|, where C - small number.
 * Returns:
 *  hi
 *
 * Description:
 *  Input value is represented as x = k*(Pi/2) + r,
 *  where:
 *    k is integer (returned value is k%4);
 *    |r| <= Pi/4, and r = d[0]+d[1],
 *    and sign of x processed as well as its magnitude.
 *
 *  Two sub-domains:
 *  1. |x| < 2^30: use Cody & Waite algorithm, in extended precision;
 *  2. |x| > 2^30: use Payne & Hanek algorithm, in extended precision.
 */

#define BIAS_80F 16383
#define _XPD_ ,0x0000,0x0000,0x0000

/* macros to form a double value in hex representation (unsigned int type) */
#define DOUBLE_HEX(hi,lo) 0x##lo,0x##hi /* LITTLE_ENDIAN */
#define LDOUBLE_HEX(w4,w3,w2,w1,w0) 0x##w0,0x##w1,0x##w2,0x##w3,0x##w4 _XPD_ /* LITTLE_ENDIAN */

#define VOLATILE_80
#define ALIGN(align) __attribute__ ((aligned (align)))

typedef struct {
    unsigned long long significand;
    unsigned int exponent:15;
    unsigned int sign:1;
    unsigned int unused0 : 16;
    unsigned int unused1 : 32;
} bits_80f;

/* miscellaneous constants */

static const double zero_none[2] = { 0.0, -1.0 };

#define TWO_12H (*(const double *)_TWO_12H)
static const unsigned int _TWO_12H[] ALIGN(8) = { DOUBLE_HEX(40b80000,00000000) };   /* 2^12+2^11 */
#define TWO_32H (*(const double *)_TWO_32H)
static const unsigned int _TWO_32H[] ALIGN(8) = { DOUBLE_HEX(41f80000,00000000) };   /* 2^32+2^31 */
#define SCALE (*(const double *)_SCALE)
static const unsigned int _SCALE[] ALIGN(8) = { DOUBLE_HEX(32600000,00000000) }; /* 2^(-217) = 2^(-31*7) */

/* pi/4 constants */

#define pi04_2d ((const double *)_pi04_2d)
static const unsigned int _pi04_2d[] ALIGN(8) = {    /* pi/4, 32x2 */
     DOUBLE_HEX(3fe921fb,54400000)  /*  .785398163367062807083129882812500000000000000000000000000000e-00 */
    ,DOUBLE_HEX(3dc0b461,1a626331)  /*  .303855025325309630073757210492923498437764552437361480769541e-10 */
};

#define pi04_3d ((const double *)_pi04_3d)
static const unsigned int _pi04_3d[] ALIGN(8) = {    /* pi/4, 47x3 */
     DOUBLE_HEX(3fe921fb,54442d00)  /*  .785398163397445614464231766760349273681640625000000000000000e-00 */
    ,DOUBLE_HEX(3ce84698,98cc5180)  /*  .269515142907906579494247111797478885049025865150440495199291e-14 */
    ,DOUBLE_HEX(b9dfc8f8,cbb5bf6c)  /* -.626849510346624994510184637043694046145457437919024410208081e-29 */
};

#define pi04_5d ((const double *)_pi04_5d)
static const unsigned int _pi04_5d[] ALIGN(8) = {    /* pi/4, 32x5 */
     DOUBLE_HEX(3fe921fb,54400000)  /* .7853981633670628070831298828125000000000000000000000000000000e-00 */
    ,DOUBLE_HEX(3dc0b461,1a600000)  /* .3038550253151982988297774568309250753372907638549804687500000e-10 */
    ,DOUBLE_HEX(3b93198a,2e000000)  /* .1011133124355583227897997811827585350891167692566341429483145e-20 */
    ,DOUBLE_HEX(396b839a,25200000)  /* .4239213830174114661430218740575102487250211784563344846134318e-31 */
    ,DOUBLE_HEX(37027044,533e63a0)  /* .1033516054913199411824845152580242234707331597899093943696227e-42 */
};

/* 4/pi constants */

#define _4onpi_d (*(const double *)__4onpi_d)
static const unsigned int __4onpi_d[] ALIGN(8) = {   /* 4/pi */
    DOUBLE_HEX(3ff45f30,6dc9c883)   /* 1.27323954473516276486577680771006271243095397949218750 */
};

#define _4onpi_31l ((const long double *)__4onpi_31l)
static const unsigned short __4onpi_31l[] ALIGN(16) = {   /* 4/pi, scaled by 2^(31*7), split by 31 bits */
/*  0*/  LDOUBLE_HEX(0000,0000,0000,0000,0000)  /* dummy */
/*  1*/ ,LDOUBLE_HEX(40d8,a2f9,836e,0000,0000)  /* 2.681755485681952834601226146146834117146940147246239188530789e+0065 */
/*  2*/ ,LDOUBLE_HEX(40b7,9c88,2a50,0000,0000)  /* 2.998555400847151729764757899853169791524063316070419660800000e+0055 */
/*  3*/ ,LDOUBLE_HEX(4099,fe13,abe8,0000,0000)  /* 4.532882328420096180171274436867472608402918604800000000000000e+0046 */
/*  4*/ ,LDOUBLE_HEX(4079,fa9a,6ee0,0000,0000)  /* 1.040965192108150426749074114550851174400000000000000000000000e+0037 */
/*  5*/ ,LDOUBLE_HEX(4058,db62,9580,0000,0000)  /* 1.060881216796656136278245376000000000000000000000000000000000e+0027 */
/*  6*/ ,LDOUBLE_HEX(403d,c9e2,1c82,0000,0000)  /* 7.273610545515724800000000000000000000000000000000000000000000e+0018 */
/*  7*/ ,LDOUBLE_HEX(4019,ff28,b1c0,0000,0000)  /* 1.337767820000000000000000000000000000000000000000000000000000e+0008 */
/*  8*/ ,LDOUBLE_HEX(3ffe,af7a,ef14,0000,0000)  /* 6.854695724323391914367675781250000000000000000000000000000000e-0001 */
/*  9*/ ,LDOUBLE_HEX(3fdf,c36e,48dc,0000,0000)  /* 3.554865529546702429541937817703001201152801513671875000000000e-0010 */
/* 10*/ ,LDOUBLE_HEX(3fbe,e909,3740,0000,0000)  /* 4.934729232545426713565924023863901171083057306532282382249832e-0020 */
/* 11*/ ,LDOUBLE_HEX(3fa2,b801,924a,0000,0000)  /* 1.451552489391147235598098827440735610676050173837014220859509e-0028 */
/* 12*/ ,LDOUBLE_HEX(3f83,dd41,3a32,0000,0000)  /* 8.127617522545783945465534150740217914783903883289875278665331e-0038 */
/* 13*/ ,LDOUBLE_HEX(3f62,873f,8778,0000,0000)  /* 5.783784182329095533532184909902121152931328737951877344877212e-0048 */
/* 14*/ ,LDOUBLE_HEX(3f44,b1cb,1298,0000,0000)  /* 7.081024809021864074177061378353359648990646709432940059311943e-0057 */
/* 15*/ ,LDOUBLE_HEX(3f26,9cfb,a208,0000,0000)  /* 5.822820908690491759905567253710591866551619832427830647935779e-0066 */
/* 16*/ ,LDOUBLE_HEX(3f06,d7d4,baec,0000,0000)  /* 1.863952698522570303825442801652407985410823100973571417219222e-0075 */
/* 17*/ ,LDOUBLE_HEX(3ee7,8909,d338,0000,0000)  /* 5.511038662968677115965437167689780303774891216520473429086808e-0085 */
/* 18*/ ,LDOUBLE_HEX(3ec7,e04d,68b8,0000,0000)  /* 2.100222289360584481213661966707142600936469662594508795363693e-0094 */
/* 19*/ ,LDOUBLE_HEX(3eaa,df90,4e64,0000,0000)  /* 3.899085782521861408214026855428168192495286468823610718475959e-0103 */
/* 20*/ ,LDOUBLE_HEX(3e89,eb1c,c1a8,0000,0000)  /* 4.773611349391312535888872569787359782437872585552380967945782e-0113 */
/* 21*/ ,LDOUBLE_HEX(3e6a,ce7d,2720,0000,0000)  /* 1.952263575238593692472297952913600491441966276720688462172563e-0122 */
/* 22*/ ,LDOUBLE_HEX(3e4b,8bf1,77b8,0000,0000)  /* 6.161178961344953715783180708404192518572042433378088840391154e-0132 */
/* 23*/ ,LDOUBLE_HEX(3e2e,e4a0,ec7e,0000,0000)  /* 1.874876302631195533370901942093950971691941797715927982101801e-0140 */
/* 24*/ ,LDOUBLE_HEX(3e0f,f12f,ffbc,0000,0000)  /* 9.210155647369374059010231921675847274611816500434440201673553e-0150 */
/* 25*/ ,LDOUBLE_HEX(3deb,b301,fdc0,0000,0000)  /* 9.947277628704842686529000547035089784059448123384322367579382e-0161 */
/* 26*/ ,LDOUBLE_HEX(3dd1,9788,c5ac,0000,0000)  /* 1.254767612059374389412162459325934363329560255480021414827149e-0168 */
/* 27*/ ,LDOUBLE_HEX(3db2,829b,47da,0000,0000)  /* 5.036021692645222078484517168939642703402212975662418721131931e-0178 */
/* 28*/ ,LDOUBLE_HEX(3d93,a6cf,d9e4,0000,0000)  /* 2.995158405021203755718469071904231574654347401460947431763942e-0187 */
/* 29*/ ,LDOUBLE_HEX(3d73,f961,36e8,0000,0000)  /* 1.042543133675461470807420313022826027201298924058101791069800e-0196 */
/* 30*/ ,LDOUBLE_HEX(3d54,f463,f668,0000,0000)  /* 4.757592860366800457906261342489524780790575069056780017023007e-0206 */
/* 31*/ ,LDOUBLE_HEX(3d35,f2ff,5168,0000,0000)  /* 2.202797822629014884554877278148182656738258697147123417217974e-0215 */
/* 32*/ ,LDOUBLE_HEX(3d17,ea4f,758e,0000,0000)  /* 1.978175408175948637814336421986762357039935282124027094544284e-0224 */
/* 33*/ ,LDOUBLE_HEX(3cf8,ebe5,f17a,0000,0000)  /* 9.274020404233314279280126729294793104566333350779107322443640e-0234 */
/* 34*/ ,LDOUBLE_HEX(3cd9,9e83,9cfa,0000,0000)  /* 2.901893454060845918648061423948507323593839835812388331374236e-0243 */
/* 35*/ ,LDOUBLE_HEX(3cba,e294,a4ba,0000,0000)  /* 1.931552229113347900614414269260619568773523610126493371005764e-0252 */
/* 36*/ ,LDOUBLE_HEX(3c9a,9afe,d7ec,0000,0000)  /* 3.076400249698286962289757777535188751026228983661112122805934e-0262 */
/* 37*/ ,LDOUBLE_HEX(3c79,8fc6,ae80,0000,0000)  /* 3.322160734595176418207650524826460758975526385234489095714439e-0272 */
/* 38*/ ,LDOUBLE_HEX(3c5c,8560,3304,0000,0000)  /* 5.740385177384701432600346775996527410943325637255512284039555e-0281 */
/* 39*/ ,LDOUBLE_HEX(3c3b,df8f,6d70,0000,0000)  /* 1.120131017324955744654738558963816735042484197431194675185339e-0290 */
/* 40*/ ,LDOUBLE_HEX(3c1e,afc3,3ef0,0000,0000)  /* 1.640329138947202592794635113428016412850173394904117840079412e-0299 */
/* 41*/ ,LDOUBLE_HEX(3bfe,826b,d0d8,0000,0000)  /* 2.833952183536812715695895445731992402558855645425552595219661e-0309 */
/* 42*/ ,LDOUBLE_HEX(3bdd,ed4f,1c80,0000,0000)  /* 6.003007410780709445059231328521992025001443679509236337298584e-0319 */
/* 43*/ ,LDOUBLE_HEX(3bc1,b0af,730c,0000,0000)  /* 1.665003989716126535993392500579592119866681725861825075217698e-0327 */
/* 44*/ ,LDOUBLE_HEX(3ba2,c219,6660,0000,0000)  /* 8.517431709312100267779051444365074092175390816153049966666913e-0337 */
/* 45*/ ,LDOUBLE_HEX(3b83,abe2,940c,0000,0000)  /* 3.512312645291607753115693933500706416067391615976397635860319e-0346 */
/* 46*/ ,LDOUBLE_HEX(3b64,8408,dffc,0000,0000)  /* 1.256357160673366920603240110131310131194691128206450935204947e-0355 */
/* 47*/ ,LDOUBLE_HEX(3b45,c402,6b98,0000,0000)  /* 8.685049467455749162247770773283923939999159367985890991068879e-0365 */
/* 48*/ ,LDOUBLE_HEX(3b26,9cc4,1818,0000,0000)  /* 3.234575316820223236082181600991473413536763570432057775845589e-0374 */
/* 49*/ ,LDOUBLE_HEX(3b05,aab6,5390,0000,0000)  /* 4.100531435295001064651143641106688810207515367189580545239542e-0384 */
/* 50*/ ,LDOUBLE_HEX(3ae9,d464,b070,0000,0000)  /* 1.900537852545528161395732681454469182895517523261005428832438e-0392 */
/* 51*/ ,LDOUBLE_HEX(3aca,9ef0,231a,0000,0000)  /* 6.622678175570520597049575876992567140164221283283373377170458e-0402 */
/* 52*/ ,LDOUBLE_HEX(3aaa,d1f1,0670,0000,0000)  /* 2.036780620549997614714934931959144289303998614795363058095878e-0411 */
/* 53*/ ,LDOUBLE_HEX(3a8a,d9f3,7738,0000,0000)  /* 4.923172230494602059088289655605244814824622442729577366586274e-0421 */
/* 54*/ ,LDOUBLE_HEX(3a6c,8092,a834,0000,0000)  /* 2.704802672273785608767665840854748321942527029073538263120668e-0430 */
/* 55*/ ,LDOUBLE_HEX(3a4d,ce23,b45c,0000,0000)  /* 2.019377481739723933347346445728650736556999919276142754308283e-0439 */
/* 56*/ ,LDOUBLE_HEX(3a2d,b0e5,36e8,0000,0000)  /* 4.034715852621462282613989371180485986546031021189823454330863e-0449 */
/* 57*/ ,LDOUBLE_HEX(3a10,af44,d156,0000,0000)  /* 7.446141374431185413875528021780337436531103470275356839707480e-0458 */
/* 58*/ ,LDOUBLE_HEX(39f1,8c82,9f52,0000,0000)  /* 2.779743591633295115161958424805826406901401048285772618498525e-0467 */
/* 59*/ ,LDOUBLE_HEX(39d1,ff83,829c,0000,0000)  /* 1.176931266910686468990349764377548721002825117889836144787437e-0476 */
/* 60*/ ,LDOUBLE_HEX(39b3,efc6,7d06,0000,0000)  /* 1.028588216965028191918385360231311368889004454648403099209490e-0485 */
/* 61*/ ,LDOUBLE_HEX(3992,b0b7,93e0,0000,0000)  /* 8.825224041761362161365014372915103469704528089597316093923779e-0496 */
/* 62*/ ,LDOUBLE_HEX(3975,c193,edde,0000,0000)  /* 1.800665099124654436611104299805867904780517918120295784664853e-0504 */
/* 63*/ ,LDOUBLE_HEX(3952,cf49,bbc0,0000,0000)  /* 5.611793040015334298974270605911362797039073360387132767241764e-0515 */
/* 64*/ ,LDOUBLE_HEX(3937,d63c,bdf0,0000,0000)  /* 4.321281812309594720967785793258548142489348709994233811095616e-0523 */
/* 65*/ ,LDOUBLE_HEX(3918,9f3a,1f34,0000,0000)  /* 1.495562303798319324534019406518764377338605087845718802452700e-0532 */
/* 66*/ ,LDOUBLE_HEX(38f9,e579,3f8e,0000,0000)  /* 1.003669217819345840702088046336224630066458848386784653863777e-0541 */
/* 67*/ ,LDOUBLE_HEX(38d9,c3f8,90c8,0000,0000)  /* 1.995674387240667334054456606095384023415101830699243564770428e-0551 */
/* 68*/ ,LDOUBLE_HEX(38b7,f8f8,48c0,0000,0000)  /* 1.475791103358781591292121873240544242443700276043556881496379e-0561 */
/* 69*/ ,LDOUBLE_HEX(389c,afa6,ed56,0000,0000)  /* 7.757494469478379305686194076020001788990899852207847667853220e-0570 */
/* 70*/ ,LDOUBLE_HEX(387d,b969,8218,0000,0000)  /* 3.813085414862534678068196881476581304103968275046986352213007e-0579 */
/* 71*/ ,LDOUBLE_HEX(385e,ec57,1852,0000,0000)  /* 2.263320851943178835197044160774772657491758466322002017111620e-0588 */
/* 72*/ ,LDOUBLE_HEX(383e,d674,670c,0000,0000)  /* 4.781720648426822482567097577249621438926886718654671889378786e-0598 */
/* 73*/ ,LDOUBLE_HEX(3820,c2c4,ad40,0000,0000)  /* 4.044519847896219581689790904875050220417986391774823891813650e-0607 */
/* 74*/ ,LDOUBLE_HEX(3801,a696,2e80,0000,0000)  /* 1.610864789985686596524169551250905027486720408179788516717091e-0616 */
/* 75*/ ,LDOUBLE_HEX(37dc,c467,d800,0000,0000)  /* 1.381856724787643494807625864093661061940326286711433581936591e-0627 */
/* 76*/ ,LDOUBLE_HEX(37c3,c5ae,3c72,0000,0000)  /* 4.144987668308343902129561139914470938016821857912328588456333e-0635 */
/* 77*/ ,LDOUBLE_HEX(37a4,ac69,b006,0000,0000)  /* 1.683444938353610734703622971063408627058824939015545056648597e-0644 */
/* 78*/ ,LDOUBLE_HEX(3782,8cdf,34a0,0000,0000)  /* 8.006336047117010439305185753858496674422757803068075080455196e-0655 */
/* 79*/ ,LDOUBLE_HEX(3766,d25e,9ed2,0000,0000)  /* 4.454024254013994133824252670012937070189464536540764495622046e-0663 */
/* 80*/ ,LDOUBLE_HEX(3747,aaaa,6fec,0000,0000)  /* 1.682620336548980068034663124886956632890035455191372233027860e-0672 */
/* 81*/ ,LDOUBLE_HEX(3726,fb5c,6040,0000,0000)  /* 2.885011022625553541693757347644447954008995308388407409605204e-0682 */
/* 82*/ ,LDOUBLE_HEX(3708,a3fc,764c,0000,0000)  /* 1.752900362911331883182460796300660943573478446213054915048956e-0691 */
/* 83*/ ,LDOUBLE_HEX(36e9,954e,b254,0000,0000)  /* 7.431927659646681236658235324620754061141726820451018237407334e-0701 */
/* 84*/ ,LDOUBLE_HEX(36ca,f5dc,3e1c,0000,0000)  /* 5.698738091599409062369918753089675337115707075994243977960468e-0710 */
/* 85*/ ,LDOUBLE_HEX(36ac,c635,7b06,0000,0000)  /* 4.278716878098401652200972178194590641772395077790182996902554e-0719 */
/* 86*/ ,LDOUBLE_HEX(368d,d738,a8ba,0000,0000)  /* 2.163444780242635357107157358932598276706009199740206743305061e-0728 */
/* 87*/ ,LDOUBLE_HEX(366d,b24e,06cc,0000,0000)  /* 4.173146843573438887486512556762771813477754614777477541493344e-0738 */
/* 88*/ ,LDOUBLE_HEX(364f,ac76,7108,0000,0000)  /* 3.759203786665369529758391609772369439865061233692872232681742e-0747 */
/* 89*/ ,LDOUBLE_HEX(3630,a7cb,2324,0000,0000)  /* 1.703123196219798335454451591982606269466974729298847268196558e-0756 */
/* 90*/ ,LDOUBLE_HEX(360f,ef15,ac40,0000,0000)  /* 2.825093152515564759022205539917682318495080888707386656687526e-0766 */
/* 91*/ ,LDOUBLE_HEX(35f2,d516,ae46,0000,0000)  /* 4.689984379754771827787530727265497870987637389896944226255665e-0775 */
/* 92*/ ,LDOUBLE_HEX(35d3,e003,615e,0000,0000)  /* 2.295910424309454696268386408508754707600870108009479401613893e-0784 */
/* 93*/ ,LDOUBLE_HEX(35b1,efe7,0cf0,0000,0000)  /* 1.431186783468271396084505711504296039017900325157261119801631e-0794 */
/* 94*/ ,LDOUBLE_HEX(3595,f98c,fb50,0000,0000)  /* 5.546006822803182623845491911063147961387599124806665917061081e-0803 */
/* 95*/ ,LDOUBLE_HEX(3575,f333,0abc,0000,0000)  /* 1.258415899929730818489651174619116626365179232095730338436182e-0812 */
/* 96*/ ,LDOUBLE_HEX(3555,ca3f,dd60,0000,0000)  /* 2.436628184906478077019088206139880237286427090157386536653723e-0822 */
/* 97*/ ,LDOUBLE_HEX(3538,d87f,7eb6,0000,0000)  /* 4.858317985206720675736611054420460139950032692050360345238557e-0831 */
/* 98*/ ,LDOUBLE_HEX(3519,b291,44f4,0000,0000)  /* 1.865969088829134375655685659296897377627082054206516090924679e-0840 */
/* 99*/ ,LDOUBLE_HEX(34f6,c982,ff80,0000,0000)  /* 6.128477139338924914814143275506001734704254655082827431565185e-0851 */
/*100*/ ,LDOUBLE_HEX(34db,d9b8,9de0,0000,0000)  /* 4.933362566956081625926460078266295697446603940849262578915341e-0859 */
/*101*/ ,LDOUBLE_HEX(34bc,9366,cd42,0000,0000)  /* 1.555301671978732556227613706462763495359010877209005976374556e-0868 */
/*102*/ ,LDOUBLE_HEX(349d,faee,bef0,0000,0000)  /* 1.232933786049357126682301850197413969149718061836838408595593e-0877 */
/*103*/ ,LDOUBLE_HEX(347d,b6f1,dac4,0000,0000)  /* 2.092872357123501721443840269711470523410269413852389830445555e-0887 */
/*104*/ ,LDOUBLE_HEX(345d,94de,f140,0000,0000)  /* 3.965269098553863599929456583486667784652014473386360016837357e-0897 */
/*105*/ ,LDOUBLE_HEX(343e,8b4b,a218,0000,0000)  /* 1.727706565038584539330481763445048289406751388066257678685540e-0906 */
/*106*/ ,LDOUBLE_HEX(341e,a135,6380,0000,0000)  /* 4.655445925516275952488036950346858030730097438464340730162033e-0916 */
/*107*/ ,LDOUBLE_HEX(3402,8cb2,b184,0000,0000)  /* 1.513635577068482699618113282427247025693221886530748892267440e-0924 */
/*108*/ ,LDOUBLE_HEX(33e3,dc61,196e,0000,0000)  /* 1.104013184698216626588715344229205238796382085456618338327998e-0933 */
/*109*/ ,LDOUBLE_HEX(33c4,de05,0c00,0000,0000)  /* 5.179229383856899000108515389745615579984756469321211016210105e-0943 */
/*110*/ ,LDOUBLE_HEX(33a5,bd38,ef9a,0000,0000)  /* 2.055494415664687400559240923759367934164832309400823473710320e-0952 */
/*111*/ ,LDOUBLE_HEX(3385,df00,c1a0,0000,0000)  /* 5.640199899537055384077558832355669447937716422655041370656092e-0962 */
/*112*/ ,LDOUBLE_HEX(3365,9973,1090,0000,0000)  /* 9.036274500439167760544513905990944678630547096870127088141797e-0972 */
/*113*/ ,LDOUBLE_HEX(3348,8301,4882,0000,0000)  /* 1.436951325836848143386592108142855995352533190884933334845922e-0980 */
/*114*/ ,LDOUBLE_HEX(3329,adc7,7abe,0000,0000)  /* 8.876098513571524633474922576560972205024717061634880632635839e-0990 */
/*115*/ ,LDOUBLE_HEX(330a,ec2b,7cba,0000,0000)  /* 5.617187322420553620956750688754956829592490553112868170256853e-0999 */
/*116*/ ,LDOUBLE_HEX(32e9,8f21,a520,0000,0000)  /* 3.963141272250030511840604330804927940184794158272360083922527e-1009 */
/*117*/ ,LDOUBLE_HEX(32cc,8d36,710c,0000,0000)  /* 7.282966682054201093860314196493079115555629364840055547656100e-1018 */
/*118*/ ,LDOUBLE_HEX(32ad,c6ed,5212,0000,0000)  /* 4.777477193681704124618148661465511299009588624480246036115038e-1027 */
/*119*/ ,LDOUBLE_HEX(328d,fd76,7308,0000,0000)  /* 1.417290902241672039538791696470368849271180261253038871822020e-1036 */
/*120*/ ,LDOUBLE_HEX(326f,d548,5014,0000,0000)  /* 1.110708896527839867626892257743069527764614891345900955049782e-1045 */
/*121*/ ,LDOUBLE_HEX(3250,b499,d3f2,0000,0000)  /* 4.379606647472791020107302582048881770477343138629591261258550e-1055 */
/*122*/ ,LDOUBLE_HEX(3230,a606,7f74,0000,0000)  /* 9.374105718720367441644386109206211989251758419474073093010443e-1065 */
/*123*/ ,LDOUBLE_HEX(3212,d720,f0a8,0000,0000)  /* 1.131237477762808027154645572158743206151263308069917437047324e-1073 */
/*124*/ ,LDOUBLE_HEX(31f2,e20f,185c,0000,0000)  /* 2.767689478381315025097796516298263408461785028601086510880734e-1083 */
/*125*/ ,LDOUBLE_HEX(31d4,8738,a5a8,0000,0000)  /* 1.541845268497461876760483162507174943799923434919511674452382e-1092 */
/*126*/ ,LDOUBLE_HEX(31b4,cafb,dd74,0000,0000)  /* 5.388865903917694407928626329989979368967410838451705554856556e-1102 */
/*127*/ ,LDOUBLE_HEX(3196,bd8e,98b6,0000,0000)  /* 4.686794301588764294382906029154933286647302893689634709028382e-1111 */
/*128*/ ,LDOUBLE_HEX(3177,977f,e9de,0000,0000)  /* 1.744286522099964478456279569193811356624111849312666349636732e-1120 */
/*129*/ ,LDOUBLE_HEX(3158,818d,67c0,0000,0000)  /* 6.945789831072683820815264019656316517782931550751442614913555e-1130 */
/*130*/ ,LDOUBLE_HEX(3139,9322,e52a,0000,0000)  /* 3.673383638410887141297298476715762684875968446489156456455323e-1139 */
/*131*/ ,LDOUBLE_HEX(3119,9b6c,e568,0000,0000)  /* 9.034590926369074039403158545082437789862848725247576690855362e-1149 */
/*132*/ ,LDOUBLE_HEX(30fa,aa0a,2358,0000,0000)  /* 4.602637606557342193512368163965779507364564484460106125647222e-1158 */
/*133*/ ,LDOUBLE_HEX(30d9,e13b,e480,0000,0000)  /* 7.097423318892777696155147241939775471133954744218014441217072e-1168 */
/*134*/ ,LDOUBLE_HEX(30bd,90a1,3024,0000,0000)  /* 1.697791185445537137105184414913003475104613549199659675455940e-1176 */
/*135*/ ,LDOUBLE_HEX(309d,da30,9620,0000,0000)  /* 5.963504225818565794305343574819835593862138215809844579818198e-1186 */
/*136*/ ,LDOUBLE_HEX(307f,b388,898a,0000,0000)  /* 4.569959667110977035712674989179758345776669925794622076117880e-1195 */
/*137*/ ,LDOUBLE_HEX(3060,c891,b24c,0000,0000)  /* 2.377395596449057240481709177089693500691549227157794252892305e-1204 */
/*138*/ ,LDOUBLE_HEX(3041,f98b,8056,0000,0000)  /* 1.377387857268770708640496606562348029301944657382240477072811e-1213 */
/*139*/ ,LDOUBLE_HEX(3021,a1ea,72a4,0000,0000)  /* 2.080831457588353305433475133019544575643569028012228286139387e-1223 */
/*140*/ ,LDOUBLE_HEX(3001,9488,6af8,0000,0000)  /* 4.444371873141919403935553162917439884563787009292871131536521e-1233 */
/*141*/ ,LDOUBLE_HEX(2fe4,dfcb,e00c,0000,0000)  /* 1.247300164263653461518172414612117838690494002655963959288881e-1241 */
/*142*/ ,LDOUBLE_HEX(2fc4,c941,eeec,0000,0000)  /* 2.611621481371153384090768999991054590721073390444297020105138e-1251 */
/*143*/ ,LDOUBLE_HEX(2fa4,e70f,53e0,0000,0000)  /* 6.981081636436945962431605175688197418749818799140835484884648e-1261 */
/*144*/ ,LDOUBLE_HEX(2f85,9c07,8f60,0000,0000)  /* 2.195205206592934734228141050177869646916243302002149097817023e-1270 */
/*145*/ ,LDOUBLE_HEX(2f68,c3e7,b328,0000,0000)  /* 5.133861374433103074381525944311838759071638657618462686425007e-1279 */
/*146*/ ,LDOUBLE_HEX(2f48,f8c7,9404,0000,0000)  /* 1.517935623846949125872803241303281130038798481466980267559738e-1288 */
/*147*/ ,LDOUBLE_HEX(2f29,c99f,38e0,0000,0000)  /* 5.728574352839368531236411930884176399952912699392173284592665e-1298 */
/*148*/ ,LDOUBLE_HEX(2f09,d984,9778,0000,0000)  /* 1.438944128546611564949987940983023709422799772789626143780151e-1307 */
/*149*/ ,LDOUBLE_HEX(2eea,d142,e700,0000,0000)  /* 6.446262522932143961080696420706288506978148801211689134619271e-1317 */
/*150*/ ,LDOUBLE_HEX(2ecd,9443,d904,0000,0000)  /* 8.507223599502116505397598531819849639851578485024466975895261e-1326 */
/*151*/ ,LDOUBLE_HEX(2eae,ae7e,d4ba,0000,0000)  /* 4.662333308727315794650729040206823222257787430717947022848778e-1335 */
/*152*/ ,LDOUBLE_HEX(2e8f,8524,8e5e,0000,0000)  /* 1.656559592455011720931118292038779369148124319754706112166667e-1344 */
/*153*/ ,LDOUBLE_HEX(2e6e,c9ed,b550,0000,0000)  /* 2.924807256506369236675329109360735997684323080755615265840381e-1354 */
/*154*/ ,LDOUBLE_HEX(2e51,8648,53b8,0000,0000)  /* 3.622840065785160014672599817919684696107937038277122032318808e-1363 */
/*155*/ ,LDOUBLE_HEX(2e32,87f9,dae4,0000,0000)  /* 1.708291732376330022381594638468636060677476714251648750829845e-1372 */
/*156*/ ,LDOUBLE_HEX(2e13,d966,2942,0000,0000)  /* 1.271825709989847461304136415132958452459782350235880380238764e-1381 */
/*157*/ ,LDOUBLE_HEX(2df3,cf3c,4f28,0000,0000)  /* 2.822762827201029772759985715120781634292158091738796521323707e-1391 */
/*158*/ ,LDOUBLE_HEX(2dd1,c4ef,fa40,0000,0000)  /* 1.561415323739370725023688350411343655387329416634281214147255e-1401 */
/*159*/ ,LDOUBLE_HEX(2db5,bca7,4424,0000,0000)  /* 5.572049042997919252943719603001819385012970438847001316434672e-1410 */
/*160*/ ,LDOUBLE_HEX(2d97,cdc5,2e62,0000,0000)  /* 5.660216247837401815574392121984231419222149301352067568017240e-1419 */
/*161*/ ,LDOUBLE_HEX(2d78,996b,ed88,0000,0000)  /* 1.965201488476453849365752005585357556596451890473995843421710e-1428 */
/*162*/ ,LDOUBLE_HEX(2d56,d97d,7c30,0000,0000)  /* 1.621587502571221317324898180108114531484170702385727781469596e-1438 */
/*163*/ ,LDOUBLE_HEX(2d3a,bf6e,ed26,0000,0000)  /* 5.317143296870287940773157201849606646297627821389218932348499e-1447 */
/*164*/ ,LDOUBLE_HEX(2d1a,921b,2918,0000,0000)  /* 9.448640807342304007079129050266062011241511763294933892149552e-1457 */
/*165*/ ,LDOUBLE_HEX(2cfb,e84e,4e24,0000,0000)  /* 6.995702628800152560862962246867025378505623851290188924484441e-1466 */
/*166*/ ,LDOUBLE_HEX(2cdd,92ec,6dc0,0000,0000)  /* 4.120621085002460325058192142547114808965764737310549422186008e-1475 */
/*167*/ ,LDOUBLE_HEX(2cbd,acf8,4f2c,0000,0000)  /* 1.129489211535154740950815757912863978796777361645133180495102e-1484 */
/*168*/ ,LDOUBLE_HEX(2c9e,f094,c634,0000,0000)  /* 7.315485210885991772669974892668767592466673184060261819455762e-1494 */
/*169*/ ,LDOUBLE_HEX(2c7e,e5d3,dc70,0000,0000)  /* 1.627135905686947415927580691935018202361985719700848108063702e-1503 */
/*170*/ ,LDOUBLE_HEX(2c5b,a600,2180,0000,0000)  /* 3.420434334306945560917582626049367343060187229650527117651203e-1514 */
/*171*/ ,LDOUBLE_HEX(2c3c,d680,8480,0000,0000)  /* 2.058132621284142161880422215161620575253224024483863451627172e-1523 */
/*172*/ ,LDOUBLE_HEX(2c22,d63b,8b24,0000,0000)  /* 3.063004402988373183169811013884499873052469927953333684341924e-1531 */
/*173*/ ,LDOUBLE_HEX(2c00,aa47,02e0,0000,0000)  /* 1.417095781101213181813278804907038988326937302732126751780741e-1541 */
/*174*/ ,LDOUBLE_HEX(2be3,ee84,9ad0,0000,0000)  /* 3.697374894907400381802884340356565413954677451428053853255116e-1550 */
/*175*/ ,LDOUBLE_HEX(2bc6,f699,f7dc,0000,0000)  /* 7.120296573451664009330280849025404815498926215966611520232064e-1559 */
/*176*/ ,LDOUBLE_HEX(2ba7,e490,ddde,0000,0000)  /* 3.073151868375438667788693402549020038755583214908376240263232e-1568 */
/*177*/ ,LDOUBLE_HEX(2b85,b4fd,34a0,0000,0000)  /* 1.416462699179750436659499824431718107921612805527745584749336e-1578 */
/*178*/ ,LDOUBLE_HEX(2b68,8ef6,91b4,0000,0000)  /* 2.084044686083938256329487842773573221205143657751465362935378e-1587 */
/*179*/ ,LDOUBLE_HEX(2b47,a2a7,a3e0,0000,0000)  /* 2.760323805231658502060938961221835463902933875903853629412942e-1597 */
/*180*/ ,LDOUBLE_HEX(2b2a,82b3,cce4,0000,0000)  /* 4.131488367808563493385273980764632655744834353009310623938899e-1606 */
/*181*/ ,LDOUBLE_HEX(2b0c,8207,e4be,0000,0000)  /* 3.827979759273169920789915977118921308650556964791737920579848e-1615 */
/*182*/ ,LDOUBLE_HEX(2aed,ab43,1d92,0000,0000)  /* 2.347766282697150275776258300463522944243941529566853756860694e-1624 */
/*183*/ ,LDOUBLE_HEX(2acd,f9f6,e818,0000,0000)  /* 7.978327425834900824774282110275101032758056468805146869681796e-1634 */
/*184*/ ,LDOUBLE_HEX(2aaf,ba80,ff12,0000,0000)  /* 5.543979133480652715215544416080463777722163521953697308113077e-1643 */
/*185*/ ,LDOUBLE_HEX(2a90,8529,5254,0000,0000)  /* 1.843241395651049214093340463175913996941052386692257875690609e-1652 */
/*186*/ ,LDOUBLE_HEX(2a71,e032,1b88,0000,0000)  /* 1.445111304511909352264788831524935445565154985762696903825789e-1661 */
/*187*/ ,LDOUBLE_HEX(2a50,d86d,3248,0000,0000)  /* 1.624032189666477278235782261477447371117571467587960019458334e-1671 */
/*188*/ ,LDOUBLE_HEX(2a2e,c9d5,3140,0000,0000)  /* 8.815682794985837974020978814919128914255054957017040023298942e-1682 */
/*189*/ ,LDOUBLE_HEX(2a14,bd47,14e6,0000,0000)  /* 1.231923455132975050827828361127431509002392694701867163615017e-1689 */
/*190*/ ,LDOUBLE_HEX(29f4,e544,5c10,0000,0000)  /* 3.474290294109037089077463280130030558463087017992256773283614e-1699 */
/*191*/ ,LDOUBLE_HEX(29d4,90b6,9f50,0000,0000)  /* 5.105911579563963517172368575642674582754194952897516923602350e-1709 */
/*192*/ ,LDOUBLE_HEX(29b6,ab55,9850,0000,0000)  /* 5.630010725630402082874520563661132429189166207470454677772601e-1718 */
/*193*/ ,LDOUBLE_HEX(2998,9d07,2750,0000,0000)  /* 4.805536063130139938773487398976851038456442620758099607186862e-1727 */
/*194*/ ,LDOUBLE_HEX(2973,8bbb,6700,0000,0000)  /* 3.111365692423015919084495727017947173029314459585987068606030e-1738 */
/*195*/ ,LDOUBLE_HEX(295a,ed31,5dba,0000,0000)  /* 1.574007273000387493214996914758648441859139318223451935349355e-1745 */
/*196*/ ,LDOUBLE_HEX(293a,85fe,61dc,0000,0000)  /* 2.070282552406246499522276004267814412575171014779596884455514e-1755 */
/*197*/ ,LDOUBLE_HEX(291c,d6b4,9ba2,0000,0000)  /* 3.089502879279310333557005821869847463802822930106000756301730e-1764 */
/*198*/ ,LDOUBLE_HEX(28fb,e3a5,2d30,0000,0000)  /* 3.813415392478304407952074243529327813996795260058790904724655e-1774 */
/*199*/ ,LDOUBLE_HEX(28dd,b566,6630,0000,0000)  /* 2.830042667152793490756618575268468354443141783405367676361407e-1783 */
/*200*/ ,LDOUBLE_HEX(28bf,a829,5ad4,0000,0000)  /* 2.443332207846090636406686763598210278793840747787993020664583e-1792 */
/*201*/ ,LDOUBLE_HEX(28a0,e290,89d8,0000,0000)  /* 1.532914854911641732940775046783619324825251078001805990393658e-1801 */
/*202*/ ,LDOUBLE_HEX(2881,c428,3916,0000,0000)  /* 6.180166648910853829619937303130510950312532747214810861250158e-1811 */
/*203*/ ,LDOUBLE_HEX(2860,bea4,0490,0000,0000)  /* 6.992329123221113117549972615553832430656826052895196425757796e-1821 */
/*204*/ ,LDOUBLE_HEX(2843,80ee,ee06,0000,0000)  /* 8.808491506843127312966764884250265912354066562251609973875153e-1830 */
/*205*/ ,LDOUBLE_HEX(2820,f327,fc00,0000,0000)  /* 4.834725883570394545374317233988834670838694454242202316460817e-1840 */
/*206*/ ,LDOUBLE_HEX(2800,a871,ea40,0000,0000)  /* 7.798027111585290733416502897095738485548054126554910169104708e-1850 */
/*207*/ ,LDOUBLE_HEX(27e4,9c26,63d8,0000,0000)  /* 2.692952344287642733749726241047680668253016111835549945465250e-1858 */
/*208*/ ,LDOUBLE_HEX(27c7,c0c9,07ba,0000,0000)  /* 6.192852996681637000342090971694674832572041650297721059318677e-1867 */
/*209*/ ,LDOUBLE_HEX(27a8,9797,3fa2,0000,0000)  /* 2.267565992625029365977879140267703912167213243149429032302824e-1876 */
/*210*/ ,LDOUBLE_HEX(2789,feca,21c6,0000,0000)  /* 1.774756770885167659459136879353099516571466844557113603809267e-1885 */
/*211*/ ,LDOUBLE_HEX(2768,860d,de40,0000,0000)  /* 1.087045347937360000847204663904971125561017459202139902448705e-1895 */
/*212*/ ,LDOUBLE_HEX(2749,98ce,9cc8,0000,0000)  /* 5.770068065513185432092218779572956929472549478179880886392143e-1905 */
/*213*/ ,LDOUBLE_HEX(272a,a31c,3778,0000,0000)  /* 2.868063559826707641187798936792853847258866516604493859095957e-1914 */
/*214*/ ,LDOUBLE_HEX(270b,f6e2,e778,0000,0000)  /* 2.021504898774632394495466387197952038810293770730023040067890e-1923 */
/*215*/ ,LDOUBLE_HEX(26ed,f841,59b8,0000,0000)  /* 1.893112158018648890538000285395475689982419780652427821768147e-1932 */
/*216*/ ,LDOUBLE_HEX(26cd,ad04,02e0,0000,0000)  /* 3.071874251722387804973415755500165732975168184297175912121856e-1942 */
/*217*/ ,LDOUBLE_HEX(26b0,9380,5a92,0000,0000)  /* 4.878027145980486195864141034044830169500623444083093225963103e-1951 */
/*218*/ ,LDOUBLE_HEX(268d,8886,c740,0000,0000)  /* 1.314059896545505686031526047490145523527817296208538909675850e-1961 */
/*219*/ ,LDOUBLE_HEX(266c,faf8,0680,0000,0000)  /* 2.812083663924400572040906333442629520795365222275974051922287e-1971 */
/*220*/ ,LDOUBLE_HEX(2653,897f,fb60,0000,0000)  /* 4.591567803737138973109576924493063392668390054139001317996755e-1979 */
/*221*/ ,LDOUBLE_HEX(2634,f903,8760,0000,0000)  /* 3.872148902609397788522703552277925561253702142217618389924355e-1988 */
/*222*/ ,LDOUBLE_HEX(2615,c2c8,ad2a,0000,0000)  /* 1.410430969435139403014900627246282526399527185293081975232085e-1997 */
/*223*/ ,LDOUBLE_HEX(25f6,8aef,2d86,0000,0000)  /* 4.684663176097516804176002408954430630901677724363924215567731e-2007 */
/*224*/ ,LDOUBLE_HEX(25d6,e627,1ef4,0000,0000)  /* 1.806863570232412943695268239973312405794623925245854092029517e-2016 */
/*225*/ ,LDOUBLE_HEX(25b7,8020,09e4,0000,0000)  /* 4.683964480264220351600601932318681269129972664823132573498334e-2026 */
/*226*/ ,LDOUBLE_HEX(2598,d227,7548,0000,0000)  /* 3.577562862696798524243004453609341369227948140212327321493980e-2035 */
/*227*/ ,LDOUBLE_HEX(2579,fb5b,75dc,0000,0000)  /* 1.992557752169682116891837156587139042305103042476858098788945e-2044 */
/*228*/ ,LDOUBLE_HEX(255a,c8b6,ea84,0000,0000)  /* 7.409150024101563103926683224802450496031687375728259880060990e-2054 */
/*229*/ ,LDOUBLE_HEX(253b,8145,e4d0,0000,0000)  /* 2.222120696373370893372822156749233115813698756595142793686912e-2063 */
/*230*/ ,LDOUBLE_HEX(251c,9768,3640,0000,0000)  /* 1.211925860399744155927109528464044172511283229550057415360630e-2072 */
/*231*/ ,LDOUBLE_HEX(24fe,ccec,246a,0000,0000)  /* 1.527634661333912163073349223847749508876065232373581270455741e-2081 */
/*232*/ ,LDOUBLE_HEX(24dd,a075,51d0,0000,0000)  /* 1.392522688303832398002420596266452612954859979900608847650585e-2091 */
/*233*/ ,LDOUBLE_HEX(24bf,a385,4638,0000,0000)  /* 1.321638731943326946751086843524682018771450357216382593861785e-2100 */
/*234*/ ,LDOUBLE_HEX(24a1,d776,d788,0000,0000)  /* 1.621868289405638248048616754224937710553287549967639666345352e-2109 */
/*235*/ ,LDOUBLE_HEX(2482,8997,1370,0000,0000)  /* 4.822784501054577398135016329780103231704795943845576461550074e-2119 */
/*236*/ ,LDOUBLE_HEX(2462,9b67,1e88,0000,0000)  /* 1.268263658344563433572729364887525948314897389029234300803738e-2128 */
/*237*/ ,LDOUBLE_HEX(2444,d975,6c08,0000,0000)  /* 1.652827608437604593422567925303129048740462032077332918122740e-2137 */
/*238*/ ,LDOUBLE_HEX(2422,cfc0,fdb0,0000,0000)  /* 9.191366500140653152936127023852788936493370888210669695959023e-2148 */
/*239*/ ,LDOUBLE_HEX(2406,c026,3100,0000,0000)  /* 3.166865564913452141889649901570378391688219436057538346483312e-2156 */
/*240*/ ,LDOUBLE_HEX(23e6,ae64,c5b4,0000,0000)  /* 6.692091434084243288779708279523373526867502724310603387393480e-2166 */
/*241*/ ,LDOUBLE_HEX(23c3,f687,2280,0000,0000)  /* 2.753261074890701309997410361101734819656301077324297317492665e-2176 */
/*242*/ ,LDOUBLE_HEX(23a9,9006,2de0,0000,0000)  /* 2.396827433756728012173180344444049448523043956278746485419995e-2184 */
/*243*/ ,LDOUBLE_HEX(238a,f631,24bc,0000,0000)  /* 1.907855577769850476362935136392004275393626221035279367231220e-2193 */
/*244*/ ,LDOUBLE_HEX(236b,a975,b8d4,0000,0000)  /* 6.115170554258697577933689874166325858774603653315099328384569e-2203 */
/*245*/ ,LDOUBLE_HEX(234b,b949,d9a4,0000,0000)  /* 1.556790968529435733354078771494211953483822028496157328082731e-2212 */
/*246*/ ,LDOUBLE_HEX(232d,bd39,b54e,0000,0000)  /* 1.480681007034018178269169742998538032628455745917514754114772e-2221 */
/*247*/ ,LDOUBLE_HEX(230e,9a52,4aac,0000,0000)  /* 5.623134305791336517399368793567879343621690381214845487397971e-2231 */
/*248*/ ,LDOUBLE_HEX(22ef,d085,bbbc,0000,0000)  /* 3.538139862124026198837479994995230203598724716914291839123629e-2240 */
/*249*/ ,LDOUBLE_HEX(22cf,c633,df18,0000,0000)  /* 7.830181455554400449477076859868023371748928554298112508352629e-2250 */
/*250*/ ,LDOUBLE_HEX(22af,eca5,16d0,0000,0000)  /* 2.176705523097831158984340695909538912510933809798093590101462e-2259 */
/*251*/ ,LDOUBLE_HEX(228e,df6f,f2a0,0000,0000)  /* 2.392590520542934151324667231576058879072413154007278710976114e-2269 */
/*252*/ ,LDOUBLE_HEX(2272,e86b,8c44,0000,0000)  /* 9.271425927153961988802638087878026466330356007778584651081270e-2278 */
/*253*/ ,LDOUBLE_HEX(2253,bbf4,35c0,0000,0000)  /* 3.491358237782651216488340918681638168495594767676817745253897e-2287 */
/*254*/ ,LDOUBLE_HEX(2230,dafb,0c40,0000,0000)  /* 1.183855974735231878058576694117955311966159400494608762320843e-2297 */
/*255*/ ,LDOUBLE_HEX(2216,9935,92dc,0000,0000)  /* 1.234236201557506920080939260388991856253282863637843138789569e-2305 */
/*256*/ ,LDOUBLE_HEX(21f3,bda6,0ca0,0000,0000)  /* 4.446446723950764827827173209847706907032406874909311677545304e-2316 */
/*257*/ ,LDOUBLE_HEX(21d6,a6fd,5958,0000,0000)  /* 7.292611013227005418135274457033104828852840051623129269655940e-2325 */
/*258*/ ,LDOUBLE_HEX(21b9,9d7f,a3dc,0000,0000)  /* 1.281150987851812279099343449157533739827394370515061046159956e-2333 */
/*259*/ ,LDOUBLE_HEX(2199,fcb5,79dc,0000,0000)  /* 4.786127838909688972016979235728003042914491990173463050637624e-2343 */
/*260*/ ,LDOUBLE_HEX(217b,cebb,f264,0000,0000)  /* 3.646499535192720030191039423168339259579897097804881056275848e-2352 */
/*261*/ ,LDOUBLE_HEX(215c,8308,0abe,0000,0000)  /* 1.076240832461646255227247820011618910622878716056226743224402e-2361 */
/*262*/ ,LDOUBLE_HEX(213d,b463,30ae,0000,0000)  /* 6.899387210072368347965972184225280655138361290239173160805724e-2371 */
/*263*/ ,LDOUBLE_HEX(211c,b040,6228,0000,0000)  /* 7.847785632105650283083793138253117158997625424159670389265929e-2381 */
/*264*/ ,LDOUBLE_HEX(20ff,f43b,c9b2,0000,0000)  /* 2.025581662228398083743735710586173723106866056112647045075159e-2389 */
/*265*/ ,LDOUBLE_HEX(20e0,a4b3,3d8e,0000,0000)  /* 6.360753875523341830854120327076907431820232669266783103325725e-2399 */
/*266*/ ,LDOUBLE_HEX(20c1,8dab,84e6,0000,0000)  /* 2.547783926321346247448946614085480489032834062513327674381338e-2408 */
/*267*/ ,LDOUBLE_HEX(20a1,9b74,a124,0000,0000)  /* 6.509253341019431585289682956674570692555681613044861358729154e-2418 */
/*268*/ ,LDOUBLE_HEX(2083,d497,c276,0000,0000)  /* 8.290363117654551585554605779749289276596151917910372852542765e-2427 */
/*269*/ ,LDOUBLE_HEX(2063,a466,6354,0000,0000)  /* 1.492680319398315454401388377858006629570309921429052889710931e-2436 */
/*270*/ ,LDOUBLE_HEX(2044,af0a,8654,0000,0000)  /* 7.400742878675083780188092037388315586309719657373575056054750e-2446 */
/*271*/ ,LDOUBLE_HEX(2024,fa5c,1d20,0000,0000)  /* 2.464563544720991048246037287699725719849033439451053540104767e-2455 */
/*272*/ ,LDOUBLE_HEX(2004,f3f0,bcd0,0000,0000)  /* 5.591125326828557438670025810993205051731948956947232282435609e-2465 */
/*273*/ ,LDOUBLE_HEX(1fe7,f0b6,edf0,0000,0000)  /* 1.027656633208636914875017776333649885139445922364314113822015e-2473 */
/*274*/ ,LDOUBLE_HEX(1fc9,9182,45bc,0000,0000)  /* 5.785425064971003488475324673124525441125822290603261605474102e-2483 */
/*275*/ ,LDOUBLE_HEX(1faa,dc85,e254,0000,0000)  /* 4.082909762163627006912173261836152367268426557764898323940713e-2492 */
/*276*/ ,LDOUBLE_HEX(1f8a,e9b1,b898,0000,0000)  /* 1.007404844069165669664495222471563929500625184113103660580081e-2501 */
/*277*/ ,LDOUBLE_HEX(1f6c,e6f0,0ebe,0000,0000)  /* 9.271522818728757018375749892598307248749654661670620129769176e-2511 */
/*278*/ ,LDOUBLE_HEX(1f4c,f584,a9b8,0000,0000)  /* 2.294986021314834611923767397089137252825314725863237227734486e-2520 */
/*279*/ ,LDOUBLE_HEX(1f2e,df6b,12e8,0000,0000)  /* 1.944979897695484462627010256470113920337064139926093259600475e-2529 */
/*280*/ ,LDOUBLE_HEX(1f0f,cd55,9f9e,0000,0000)  /* 8.323930520071806995299689313212619132616679704851684350416164e-2539 */
/*281*/ ,LDOUBLE_HEX(1eef,ec3a,05a0,0000,0000)  /* 2.229644473806396927225643080536524557501644805323511443174168e-2548 */
/*282*/ ,LDOUBLE_HEX(1ed1,96f8,d8e0,0000,0000)  /* 1.327099358787451110997133190193825145491281967079235522004147e-2557 */
/*283*/ ,LDOUBLE_HEX(1eb1,ccc6,3bd4,0000,0000)  /* 4.191047240868510722364714959240439853433021555252009074615389e-2567 */
/*284*/ ,LDOUBLE_HEX(1e93,b87b,4910,0000,0000)  /* 3.516415188917842921150964306304086568992511011776724690972158e-2576 */
/*285*/ ,LDOUBLE_HEX(1e73,d40b,befc,0000,0000)  /* 9.410595638851760991455063343736528747271367403943913858268657e-2586 */
/*286*/ ,LDOUBLE_HEX(1e55,a406,317e,0000,0000)  /* 6.779463190481773894985893985620209296838177623774435193852109e-2595 */
/*287*/ ,LDOUBLE_HEX(1e36,c2b2,6bb2,0000,0000)  /* 3.747284733044959220184674378357370777849401166015893779937574e-2604 */
/*288*/ ,LDOUBLE_HEX(1e17,bb78,b87e,0000,0000)  /* 1.680208092709319719202693481923835934547811364113179724121006e-2613 */
/*289*/ ,LDOUBLE_HEX(1df7,dbbd,a03c,0000,0000)  /* 4.585411979567289380891415460172771730892241989820697858945432e-2623 */
/*290*/ ,LDOUBLE_HEX(1dd9,e3c8,5b6c,0000,0000)  /* 4.426786404265599780330500632365256345662412164297237522023529e-2632 */
/*291*/ ,LDOUBLE_HEX(1dba,ca8e,8968,0000,0000)  /* 1.833094539238875534880715529121193311498992846711216563750088e-2641 */
/*292*/ ,LDOUBLE_HEX(1d9a,e6ab,c024,0000,0000)  /* 4.860388898060028587249335306160361197442298552109279846405240e-2651 */
/*293*/ ,LDOUBLE_HEX(1d7a,d4eb,4110,0000,0000)  /* 1.044558306642921699394410021031110790791227644613591217802069e-2660 */
/*294*/ ,LDOUBLE_HEX(1d5d,bdb5,a168,0000,0000)  /* 1.733554658641490028632531747268549739004388969867724937305855e-2669 */
/*295*/ ,LDOUBLE_HEX(1d3e,a5fa,012e,0000,0000)  /* 7.062613183859303371167377789592859058856790430016776003060738e-2679 */
/*296*/ ,LDOUBLE_HEX(1d1e,9c1f,6838,0000,0000)  /* 1.546766493044770730364803326899658040986376011534462599628859e-2688 */
/*297*/ ,LDOUBLE_HEX(1d00,aa76,a158,0000,0000)  /* 1.572859832848149185138503103523553864063239551642520036685983e-2697 */
/*298*/ ,LDOUBLE_HEX(1ce1,bd95,090a,0000,0000)  /* 8.145664137496274433394826767113966088596228788761519305802926e-2707 */
/*299*/ ,LDOUBLE_HEX(1cc2,8b6d,f73e,0000,0000)  /* 2.789677395089739720513589506152245870955426892101325293973820e-2716 */
/*300*/ ,LDOUBLE_HEX(1ca3,bcbf,5fda,0000,0000)  /* 1.758533384983655467398720727463919159515555681159972760261452e-2725 */
/*301*/ ,LDOUBLE_HEX(1c84,b89f,dbe8,0000,0000)  /* 8.009928274950800190638215600132025997341050194935816911463469e-2735 */
/*302*/ ,LDOUBLE_HEX(1c64,96c7,6e4c,0000,0000)  /* 1.523072191758400482771824668126690792308583774475788847616969e-2744 */
/*303*/ ,LDOUBLE_HEX(1c46,f2a4,19c2,0000,0000)  /* 2.282674055306762148877286100409197004655354500877574871453535e-2753 */
/*304*/ ,LDOUBLE_HEX(1c1e,f855,b800,0000,0000)  /* 2.124797308867201289911562417026903636946363762649041025656843e-2765 */
/*305*/ ,LDOUBLE_HEX(1c08,85ff,87fc,0000,0000)  /* 2.733498344310532719886469262047748970632187181021290930110568e-2772 */
/*306*/ ,LDOUBLE_HEX(1be9,839f,1418,0000,0000)  /* 1.250306801134818291288117058621339754196956574613697378610867e-2781 */
/*307*/ ,LDOUBLE_HEX(1bca,d9d8,6186,0000,0000)  /* 9.636255176554365331057697454082358248885372072863707187371108e-2791 */
/*308*/ ,LDOUBLE_HEX(1ba6,abaa,f500,0000,0000)  /* 1.105017665023837768327912642854506533888079060927613597578383e-2801 */
/*309*/ ,LDOUBLE_HEX(1b8c,dafe,7b36,0000,0000)  /* 2.100549111342076551081192125205168857810142973761709517232320e-2809 */
/*310*/ ,LDOUBLE_HEX(1b6c,e6d8,f394,0000,0000)  /* 5.155440055715547957203059870557303779342744910449574363774509e-2819 */
/*311*/ ,LDOUBLE_HEX(1b4e,9e55,6efc,0000,0000)  /* 3.293170277755487435846773851823128745386382586035720062745752e-2828 */
/*312*/ ,LDOUBLE_HEX(1b2e,c523,5e10,0000,0000)  /* 9.546662797538837545219611429446900347898915287656912537892575e-2838 */
/*313*/ ,LDOUBLE_HEX(1b0d,b6f9,8210,0000,0000)  /* 1.031530191688658032744525034075323048557320390281276844092878e-2847 */
/*314*/ ,LDOUBLE_HEX(1af1,96e3,9ab0,0000,0000)  /* 3.168903176075382653743947645371301780700824412747211227883157e-2856 */
/*315*/ ,LDOUBLE_HEX(1ad1,92e7,3864,0000,0000)  /* 7.183276259471914666410868369731433889620533046596193105841277e-2866 */
/*316*/ ,LDOUBLE_HEX(1ab1,dc65,9878,0000,0000)  /* 2.509209006377616857177012224081294475588062416658489803294090e-2875 */
/*317*/ ,LDOUBLE_HEX(1a94,d6cb,fa20,0000,0000)  /* 4.555010081329264435804577676172051038308921220409220070661017e-2884 */
/*318*/ ,LDOUBLE_HEX(1a70,a4e4,6c00,0000,0000)  /* 5.088409159769429236289869894212571351033810205909669365597392e-2895 */
/*319*/ ,LDOUBLE_HEX(1a53,b41b,ab40,0000,0000)  /* 1.035249543942230440048200552904579304793914670068019481455384e-2903 */
/*320*/ ,LDOUBLE_HEX(1a37,8ede,43a4,0000,0000)  /* 3.059198101138636507977254984767913531572701672206010721712710e-2912 */
/*321*/ ,LDOUBLE_HEX(1a15,9314,22e0,0000,0000)  /* 1.833165907431006810701427916691098637336674216366527568008801e-2922 */
/*322*/ ,LDOUBLE_HEX(19f8,b949,6170,0000,0000)  /* 4.301563659310185611373590578738960971816515251760830285820396e-2931 */
/*323*/ ,LDOUBLE_HEX(19d8,e056,6b00,0000,0000)  /* 1.212619064850094006094903614157803160497906806635825596357917e-2940 */
/*324*/ ,LDOUBLE_HEX(19b9,a94c,9ba8,0000,0000)  /* 4.261353368815239779105714015040053005097176974322854234268983e-2950 */
/*325*/ ,LDOUBLE_HEX(199b,aa16,faa0,0000,0000)  /* 3.987225677850425272189822860536192269176528784334547035423952e-2959 */
/*326*/ ,LDOUBLE_HEX(197d,f627,899a,0000,0000)  /* 5.374034554079146244579826304429692247736819034302412722296390e-2968 */
/*327*/ ,LDOUBLE_HEX(195d,fb70,9f20,0000,0000)  /* 1.278106834909013495217318478811289607522190498498298541562631e-2977 */
/*328*/ ,LDOUBLE_HEX(193e,c176,a4b8,0000,0000)  /* 4.579330390608197803041900145479763645369597455556938557437638e-2987 */
/*329*/ ,LDOUBLE_HEX(1920,85c3,b21c,0000,0000)  /* 2.948789981669025301344817011309306520858574694089022423146416e-2996 */
/*330*/ ,LDOUBLE_HEX(1901,9b19,50d2,0000,0000)  /* 1.592142739480811083407600906837121056568139687432183761860216e-3005 */
/*331*/ ,LDOUBLE_HEX(18e0,b708,d4b0,0000,0000)  /* 2.187340537728391819701179897193711989809023074533622034731522e-3015 */
/*332*/ ,LDOUBLE_HEX(18c1,f510,fb88,0000,0000)  /* 1.363757811846633292571716081883509226606141284003964988759834e-3024 */
/*333*/ ,LDOUBLE_HEX(18a3,dc8d,31ec,0000,0000)  /* 1.143046295706341247956831782889968550083850594832896255061197e-3033 */
/*334*/ ,LDOUBLE_HEX(1885,bff9,3c00,0000,0000)  /* 9.266063997890465925425508352522293794083931509874818137707729e-3043 */
/*335*/ ,LDOUBLE_HEX(1862,c30b,5020,0000,0000)  /* 2.739914577301496440675798076673353955902063183330138856013228e-3053 */
/*336*/ ,LDOUBLE_HEX(1844,da0c,d4f0,0000,0000)  /* 2.852728693031536695550515433910282699333696022114612158506213e-3062 */
/*337*/ ,LDOUBLE_HEX(1828,99a5,20d2,0000,0000)  /* 7.488292081725512447015838356799221028042588310907421822575065e-3071 */
/*338*/ ,LDOUBLE_HEX(1809,d159,852e,0000,0000)  /* 4.751237134954036917469779647905569568138849216512961336895276e-3080 */
/*339*/ ,LDOUBLE_HEX(17e9,97a1,7cd8,0000,0000)  /* 8.012388242706710099471431819448613800605100747974370402481238e-3090 */
/*340*/ ,LDOUBLE_HEX(17cb,997b,423a,0000,0000)  /* 7.553193887828231775412622347602472044874898955602543070525569e-3099 */
/*341*/ ,LDOUBLE_HEX(17a8,be7d,c1c0,0000,0000)  /* 2.728348302999129272040756956702339143399815187558957245232862e-3109 */
/*342*/ ,LDOUBLE_HEX(178d,dcdd,e8bc,0000,0000)  /* 2.356923184021251661521959363102568278169532350891213329933958e-3117 */
/*343*/ ,LDOUBLE_HEX(176e,ae06,8b28,0000,0000)  /* 8.647649961887522505491123562066430554952056618801175479202562e-3127 */
/*344*/ ,LDOUBLE_HEX(174f,b8d4,102e,0000,0000)  /* 4.276848163628053538276974298310658286694510968732911208847876e-3136 */
/*345*/ ,LDOUBLE_HEX(172f,aa5c,aa00,0000,0000)  /* 9.178421205505955686174516484385295343614340059164618900871991e-3146 */
/*346*/ ,LDOUBLE_HEX(170e,9fc0,51f0,0000,0000)  /* 1.001958518529351493603779324848111925358102430789716414878075e-3155 */
/*347*/ ,LDOUBLE_HEX(16f2,e181,f858,0000,0000)  /* 5.268985031162877771059513170802002920129227699692039776105882e-3164 */
/*348*/ ,LDOUBLE_HEX(16d3,8162,91a8,0000,0000)  /* 1.407731341430464452052888350654811068467393124347221627205862e-3173 */
/*349*/ ,LDOUBLE_HEX(16b1,cb6f,5f40,0000,0000)  /* 1.288373114077831204411436551152270278734490632636226163254974e-3183 */
/*350*/ ,LDOUBLE_HEX(1693,e55f,bb50,0000,0000)  /* 1.352882585940522424785318183343365259132039865841926859705177e-3192 */
/*351*/ ,LDOUBLE_HEX(1676,d895,acd2,0000,0000)  /* 2.379435000168459926045912046456830891807061832453965259260530e-3201 */
/*352*/ ,LDOUBLE_HEX(1654,97bf,ef30,0000,0000)  /* 9.704077319663385715487705005771474370861924574356542840018945e-3212 */
/*353*/ ,LDOUBLE_HEX(1633,b3d7,f700,0000,0000)  /* 1.338848522854203220959389300750922517104670956878403324857947e-3221 */
/*354*/ ,LDOUBLE_HEX(1619,a7b5,3454,0000,0000)  /* 1.860415192207997298698909461188056490308375826508810899926281e-3229 */
/*355*/ ,LDOUBLE_HEX(15f6,a929,6b00,0000,0000)  /* 5.461462544603905862787968588582965828901687747939971712762519e-3240 */
/*356*/ ,LDOUBLE_HEX(15db,89f7,9f04,0000,0000)  /* 3.318737316231351424479889301386834974216961675734559301828822e-3248 */
/*357*/ ,LDOUBLE_HEX(15bc,d985,ad78,0000,0000)  /* 2.436522830793055977011290206699601160474212799452487985968174e-3257 */
/*358*/ ,LDOUBLE_HEX(159d,ae3f,a46a,0000,0000)  /* 9.088796505578551023237292074339605080677598268984372951321490e-3267 */
/*359*/ ,LDOUBLE_HEX(157c,d0da,63a0,0000,0000)  /* 1.268201051829618998581521814827704551726177372457488357266787e-3276 */
/*360*/ ,LDOUBLE_HEX(155e,817d,5e90,0000,0000)  /* 7.322892437797263571663251144829851709246807033603850724328069e-3286 */
/*361*/ ,LDOUBLE_HEX(1540,b13f,1494,0000,0000)  /* 9.335227829098379609258506901300016941130678529874932106804551e-3295 */
/*362*/ ,LDOUBLE_HEX(1521,9c40,0090,0000,0000)  /* 3.832107324353861056211438562542398161982682558304742833931441e-3304 */
/*363*/ ,LDOUBLE_HEX(1500,cc86,dd70,0000,0000)  /* 5.839531630108400910131024483128515458358221683981401602464643e-3314 */
/*364*/ ,LDOUBLE_HEX(14e1,db6f,64f8,0000,0000)  /* 2.917454262911737935893572256724123010581855062875065063363794e-3323 */
/*365*/ ,LDOUBLE_HEX(14c3,ac17,e22c,0000,0000)  /* 2.130894610551521290232510471510751812570150505890872618752850e-3332 */
/*366*/ ,LDOUBLE_HEX(14a3,a9ad,60e0,0000,0000)  /* 4.891722375998524312606214308236238838977203051490694192514650e-3342 */
/*367*/ ,LDOUBLE_HEX(1481,d658,4640,0000,0000)  /* 3.596924351027003132249777132654327430401336565627150103415033e-3352 */
/*368*/ ,LDOUBLE_HEX(1467,a181,6490,0000,0000)  /* 4.038551054629067814815907422052123556849652128419136269348309e-3360 */
/*369*/ ,LDOUBLE_HEX(1447,aaa2,1df4,0000,0000)  /* 9.934414389596831698220262098435184549973834358158651808593529e-3370 */
/*370*/ ,LDOUBLE_HEX(1429,8f61,b94a,0000,0000)  /* 7.774500861384951506107152998894384983214378339444022257124198e-3379 */
/*371*/ ,LDOUBLE_HEX(1409,9d83,5198,0000,0000)  /* 1.988543662287678842420980140564152447842626412516405005007439e-3388 */
/*372*/ ,LDOUBLE_HEX(13eb,a818,0f7a,0000,0000)  /* 1.976383111350731283023238548735501540668424567573474362336217e-3397 */
/*373*/ ,LDOUBLE_HEX(13cc,c06c,c45e,0000,0000)  /* 1.053538224283295634707983829608349444563757827867453525322254e-3406 */
/*374*/ ,LDOUBLE_HEX(13a8,fa29,4ec0,0000,0000)  /* 1.993101766661070975849394371829882956685779311545237493521923e-3417 */
/*375*/ ,LDOUBLE_HEX(138c,8cad,6418,0000,0000)  /* 4.175355138652949783653606075257329572125770289766713630222245e-3426 */
/*376*/ ,LDOUBLE_HEX(136f,e7d1,bcc8,0000,0000)  /* 1.281589855505337311833174838824984570915561220353033252399786e-3434 */
/*377*/ ,LDOUBLE_HEX(134f,f9b0,c934,0000,0000)  /* 3.213969679919534468757160771737995745458879524504540597175710e-3444 */
/*378*/ ,LDOUBLE_HEX(1331,98df,6ce0,0000,0000)  /* 1.832609176637392261890465093072154184199561487149736811583160e-3453 */
/*379*/ ,LDOUBLE_HEX(1312,e5e9,3516,0000,0000)  /* 1.283422313434814032010885978711780007529138502830524240879370e-3462 */
/*380*/ ,LDOUBLE_HEX(12ef,ef8b,c6c0,0000,0000)  /* 3.891786225123084941983798693031380067366662394050363279642437e-3473 */
/*381*/ ,LDOUBLE_HEX(12d4,913d,af02,0000,0000)  /* 1.758082510635509374719799181103039276114672840735516632398533e-3481 */
/*382*/ ,LDOUBLE_HEX(12b5,e1d5,d230,0000,0000)  /* 1.272951949917295138732237833375584610518036771626997147708382e-3490 */
/*383*/ ,LDOUBLE_HEX(1295,c232,fba8,0000,0000)  /* 2.548635752403384498136836664429912618821269709360848633317211e-3500 */
/*384*/ ,LDOUBLE_HEX(1277,abeb,7ba4,0000,0000)  /* 2.101292972501825364210740260006784544609687059984618035472509e-3509 */
/*385*/ ,LDOUBLE_HEX(1258,c692,6e5c,0000,0000)  /* 1.130182836603762745576780912169313414463145654090569647174308e-3518 */
/*386*/ ,LDOUBLE_HEX(1239,9756,76a2,0000,0000)  /* 4.010957923115016141950968265760523977513090525620238693216888e-3528 */
/*387*/ ,LDOUBLE_HEX(1214,e423,e180,0000,0000)  /* 4.399392816057542421189223075273652699329838534754011922356730e-3539 */
/*388*/ ,LDOUBLE_HEX(11fb,90f8,8c3c,0000,0000)  /* 8.331475339798519926747641819312446607184461910640182940359587e-3547 */
/*389*/ ,LDOUBLE_HEX(11dc,9fd2,9f3c,0000,0000)  /* 4.277104104081997151132879114177440299398377696663044824361611e-3556 */
/*390*/ ,LDOUBLE_HEX(11bd,b73e,53e0,0000,0000)  /* 2.283546598774986212057036017179347908601139271189290628425391e-3565 */
/*391*/ ,LDOUBLE_HEX(119e,88d6,45be,0000,0000)  /* 7.940631851004381970607514917161746526848671831088251135528856e-3575 */
/*392*/ ,LDOUBLE_HEX(117f,8bc0,111a,0000,0000)  /* 3.776367695170485056421013720700636605091186132118203430919438e-3584 */
/*393*/ ,LDOUBLE_HEX(1160,d7ff,e26a,0000,0000)  /* 2.717969783255390877157893843392639495665469469756839373539999e-3593 */
/*394*/ ,LDOUBLE_HEX(113f,dd8d,fb60,0000,0000)  /* 3.245508332356728102093087811595719835706994876609498997607821e-3603 */
/*395*/ ,LDOUBLE_HEX(1120,c108,9370,0000,0000)  /* 1.316754292503874724807836012268530629692410102787864297902722e-3612 */
/*396*/ ,LDOUBLE_HEX(1103,8baf,9654,0000,0000)  /* 1.774821641386633810329533239696978488051966252511657722911255e-3621 */
/*397*/ ,LDOUBLE_HEX(10e4,d6b9,d6ec,0000,0000)  /* 1.270447790644983840667776944706526489414740660871791404980413e-3630 */
/*398*/ ,LDOUBLE_HEX(10c4,d7b7,23e4,0000,0000)  /* 2.971622174865039207023659660641566746462676248476252018293930e-3640 */
/*399*/ ,LDOUBLE_HEX(10a6,a847,1aa6,0000,0000)  /* 2.158933596656670170590316088305175859017191916822177355205040e-3649 */
/*400*/ ,LDOUBLE_HEX(1087,9fef,bee6,0000,0000)  /* 9.554991993406795669991734946115490289998115258881366658926494e-3659 */
/*401*/ ,LDOUBLE_HEX(1066,a6eb,26d0,0000,0000)  /* 1.160907178592603241510554253131319186730464900348148223630675e-3668 */
/*402*/ ,LDOUBLE_HEX(1049,a880,5b86,0000,0000)  /* 2.182863078363203133200454633344889461870948512582134605472907e-3677 */
/*403*/ ,LDOUBLE_HEX(1029,d971,125c,0000,0000)  /* 6.558521353208288495189119950836808256195274985650540467653816e-3687 */
/*404*/ ,LDOUBLE_HEX(100a,9d18,1f78,0000,0000)  /* 2.206446990356382863437374431995315510794531138164754371344769e-3696 */
/*405*/ ,LDOUBLE_HEX(0feb,b15b,0e84,0000,0000)  /* 1.159974721101111989485086593442913930788210835204817978686264e-3705 */
/*406*/ ,LDOUBLE_HEX(0fcc,c150,d0c0,0000,0000)  /* 5.887631575811546614993458432761679812303276365703416706436437e-3715 */
/*407*/ ,LDOUBLE_HEX(0fad,c40c,a330,0000,0000)  /* 2.780411697514625882763477048924146467261385869143266413887172e-3724 */
/*408*/ ,LDOUBLE_HEX(0f8f,fc2c,5202,0000,0000)  /* 3.330754318609419627718125286409145549623339422952949670853321e-3733 */
/*409*/ ,LDOUBLE_HEX(0f6f,ecf5,3f7c,0000,0000)  /* 7.287109763978635984468797072927788635284556851762179829622663e-3743 */
/*410*/ ,LDOUBLE_HEX(0f50,fdfd,ef44,0000,0000)  /* 3.637257030329875554343827083598738822048867770996361987167420e-3752 */
/*411*/ ,LDOUBLE_HEX(0f31,ab1b,3f6c,0000,0000)  /* 1.141012891478169258586408124939412485458091183115787224895783e-3761 */
/*412*/ ,LDOUBLE_HEX(0f11,89ec,f658,0000,0000)  /* 2.141456870314848631328243711843025305972382452433631875472824e-3771 */
/*413*/ ,LDOUBLE_HEX(0ef4,9ba8,bfc8,0000,0000)  /* 4.501623788757277958031228290555542161016827255992772063956035e-3780 */
/*414*/ ,LDOUBLE_HEX(0ed5,be21,3d40,0000,0000)  /* 2.560441679628234929720168102834275904035206021525097870421492e-3789 */
/*415*/ ,LDOUBLE_HEX(0eb6,c70d,bbc4,0000,0000)  /* 1.248259486935478813963167729186181591601746601709466939310770e-3798 */
/*416*/ ,LDOUBLE_HEX(0e96,db16,5158,0000,0000)  /* 3.198835001703944599144143466005639027634400937901537940760108e-3808 */
/*417*/ ,LDOUBLE_HEX(0e78,a8d8,b5a8,0000,0000)  /* 2.295974845028456268647408631119562687873599500278521381317171e-3817 */
/*418*/ ,LDOUBLE_HEX(0e58,b40e,cccc,0000,0000)  /* 5.700685931396714248370965843323923434361540044709946810092307e-3827 */
/*419*/ ,LDOUBLE_HEX(0e3a,cb62,448c,0000,0000)  /* 5.996968363298156958573264417894551705284938992682965064561307e-3836 */
/*420*/ ,LDOUBLE_HEX(0e1b,8aed,f12a,0000,0000)  /* 1.907566826169139649667275921511828067968776972916660946801401e-3845 */
/*421*/ ,LDOUBLE_HEX(0dfb,c59c,79d0,0000,0000)  /* 6.317382576264270784635199554928793715338959659854981854828438e-3855 */
/*422*/ ,LDOUBLE_HEX(0ddd,cdc9,06b4,0000,0000)  /* 6.126887973458516603711106566441630937650966402462284132938242e-3864 */
/*423*/ ,LDOUBLE_HEX(0dbe,a979,ae70,0000,0000)  /* 2.349645105359757784304562245103823548317407361749435737406455e-3873 */
/*424*/ ,LDOUBLE_HEX(0d9e,a8fb,317c,0000,0000)  /* 5.454744374673177254089351444405371735651122728592128824439557e-3883 */
/*425*/ ,LDOUBLE_HEX(0d7d,8a50,5fe0,0000,0000)  /* 5.197710897256277290506724375920015886297594084339265999514745e-3893 */
/*426*/ ,LDOUBLE_HEX(0d61,fdfa,70b6,0000,0000)  /* 3.555517317014445352078891376509969251010303551927025188056572e-3901 */
/*427*/ ,LDOUBLE_HEX(0d41,9dc7,1640,0000,0000)  /* 5.142708757590101423478818174346424001623724628056882597468805e-3911 */
/*428*/ ,LDOUBLE_HEX(0d23,dc50,9a9c,0000,0000)  /* 6.687907757234296519921285196916994889814705025338003158325979e-3920 */
/*429*/ ,LDOUBLE_HEX(0d04,9a9b,4fcc,0000,0000)  /* 2.185470214271526796087699010368201792462196413835613550026242e-3929 */
/*430*/ ,LDOUBLE_HEX(0ce5,8f77,7e48,0000,0000)  /* 9.443610727866688518930386382926506861683052460247546560235360e-3939 */
/*431*/ ,LDOUBLE_HEX(0cc6,d4b9,84e4,0000,0000)  /* 6.520408062535247225521885207311848860189546621034828692130167e-3948 */
/*432*/ ,LDOUBLE_HEX(0ca6,bd10,84e0,0000,0000)  /* 1.349295528129549683487992391951912903259223085359850031342161e-3957 */
/*433*/ ,LDOUBLE_HEX(0c88,c8d9,1b0a,0000,0000)  /* 1.334949440667085159800028348205889670182590011298889703035112e-3966 */
/*434*/ ,LDOUBLE_HEX(0c68,fc81,6a48,0000,0000)  /* 3.907582774753007650793721859977790657990187738450912982129850e-3976 */
/*435*/ ,LDOUBLE_HEX(0c4a,bef6,070a,0000,0000)  /* 2.752214056592271984668119388641537806884321304237774831035218e-3985 */
/*436*/ ,LDOUBLE_HEX(0c2b,f096,8a70,0000,0000)  /* 1.614661955508843081205091839175865726371364040729295969200886e-3994 */
/*437*/ ,LDOUBLE_HEX(0c0c,c994,ecc2,0000,0000)  /* 6.299830715652431330813135417622863353153262504804434491434495e-4004 */
/*438*/ ,LDOUBLE_HEX(0bea,9537,1540,0000,0000)  /* 2.714381050125743191478786437934662261826692333538372590797592e-4014 */
/*439*/ ,LDOUBLE_HEX(0bce,ab5b,1b02,0000,0000)  /* 1.161226605710887489377421546866806627261671453042327864580519e-4022 */
/*440*/ ,LDOUBLE_HEX(0bad,b0c8,5dc0,0000,0000)  /* 1.394658319008741814433784428048516253783558225569870262606297e-4032 */
/*441*/ ,LDOUBLE_HEX(0b8f,e034,c928,0000,0000)  /* 1.647311932269042229998272435544276763026727185326888196146051e-4041 */
/*442*/ ,LDOUBLE_HEX(0b71,b4b0,2d12,0000,0000)  /* 1.236397531004607885896784889017321988146025450250051652495757e-4050 */
/*443*/ ,LDOUBLE_HEX(0b52,bb94,8fc2,0000,0000)  /* 5.977034610496752501440935456692213777649011721946697446562515e-4060 */
/*444*/ ,LDOUBLE_HEX(0b33,e22f,e236,0000,0000)  /* 3.356111366528031617275751425349297218484250189124966740803199e-4069 */
/*445*/ ,LDOUBLE_HEX(0b13,be9e,b97c,0000,0000)  /* 6.585326019396280269317083327875147477515815569813016597327144e-4079 */
/*446*/ ,LDOUBLE_HEX(0af5,e16d,e1a6,0000,0000)  /* 7.253024117861513818966629393871599023244557015139517822850751e-4088 */
/*447*/ ,LDOUBLE_HEX(0ad6,baf0,d330,0000,0000)  /* 2.800805534863653261429515335780549205819959087664376506021579e-4097 */
/*448*/ ,LDOUBLE_HEX(0ab7,bbd0,c0bc,0000,0000)  /* 1.310329346622809635302493629072322803495299530151390234003001e-4106 */
/*449*/ ,LDOUBLE_HEX(0a98,dd9b,8e66,0000,0000)  /* 7.199528246044260337994568012647702371410949173440660698679339e-4116 */
/*450*/ ,LDOUBLE_HEX(0a79,f799,c95c,0000,0000)  /* 3.745772190897716597205804930474867767622683635178624953948853e-4125 */
/*451*/ ,LDOUBLE_HEX(0a55,be4c,dac0,0000,0000)  /* 4.189371487823386780936294106372520881663513893865328842496351e-4136 */
/*452*/ ,LDOUBLE_HEX(0a37,c378,afc0,0000,0000)  /* 4.007679864930195803732375289458897849761305118932723113984980e-4145 */
/*453*/ ,LDOUBLE_HEX(0a19,e341,a880,0000,0000)  /* 4.339364475133322809411118262581996082399783720312330607740805e-4154 */
/*454*/ ,LDOUBLE_HEX(09fd,81f6,c242,0000,0000)  /* 9.244697839924489578504012617231892306650108399413053276495675e-4163 */
/*455*/ ,LDOUBLE_HEX(09de,c777,7470,0000,0000)  /* 6.607091846502364153702809941421188690728647687097283712271394e-4172 */
/*456*/ ,LDOUBLE_HEX(09be,b684,62bc,0000,0000)  /* 1.407615006301715819063575335601309204476691523779804881005560e-4181 */
/*457*/ ,LDOUBLE_HEX(099f,8c58,43ac,0000,0000)  /* 5.040187115733055401018277728478435509119795970744221201574384e-4191 */
/*458*/ ,LDOUBLE_HEX(0981,f9ac,cc3c,0000,0000)  /* 8.350751521573593572918762011483343454785640251660660154228118e-4200 */
/*459*/ ,LDOUBLE_HEX(0962,b670,1526,0000,0000)  /* 2.841420652514434167039867010052512519861707982928405977687252e-4209 */
/*460*/ ,LDOUBLE_HEX(0943,df50,c9fe,0000,0000)  /* 1.619607630780760474431891709687599607737456305514998497833111e-4218 */
/*461*/ ,LDOUBLE_HEX(0924,c065,6ae6,0000,0000)  /* 6.497665143198592114383566854411985102901852241221837360742792e-4228 */
/*462*/ ,LDOUBLE_HEX(0905,cf29,b114,0000,0000)  /* 3.257939010022175550607633855045313577755666393859998252853760e-4237 */
/*463*/ ,LDOUBLE_HEX(08e4,922a,d388,0000,0000)  /* 2.676032565760984616794066163440992559248654754508198278531467e-4247 */
/*464*/ ,LDOUBLE_HEX(08c7,b926,cf54,0000,0000)  /* 6.313915948543777799401907852386322505141895537271187966776561e-4256 */
/*465*/ ,LDOUBLE_HEX(08a8,e855,3826,0000,0000)  /* 3.689369470599093734148443020925491347614492854884488701611240e-4265 */
/*466*/ ,LDOUBLE_HEX(0888,829b,e7c8,0000,0000)  /* 4.828972363645241795335883772586792435946819865296039742429476e-4275 */
/*467*/ ,LDOUBLE_HEX(086a,a903,546c,0000,0000)  /* 5.819716860309000972685049226569301029211578407040493434558250e-4284 */
/*468*/ ,LDOUBLE_HEX(0849,99cc,8768,0000,0000)  /* 6.165172266802282012678333269657333957730097220182743463837319e-4294 */
/*469*/ ,LDOUBLE_HEX(082b,f529,00ac,0000,0000)  /* 9.152545748721847756883371997993342110573031140355010885869526e-4303 */
/*470*/ ,LDOUBLE_HEX(080c,9f0b,2658,0000,0000)  /* 2.764892221014017060221253387340489846892287572944525906298552e-4312 */
/*471*/ ,LDOUBLE_HEX(07ee,9e21,fe5c,0000,0000)  /* 2.560260710955265751296940070732381736252119944236523399966742e-4321 */
/*472*/ ,LDOUBLE_HEX(07cf,9910,6da2,0000,0000)  /* 1.154000371470830452095157752736395423544262210399553107155257e-4330 */
/*473*/ ,LDOUBLE_HEX(07b0,f9b3,9220,0000,0000)  /* 8.766446679599854591787135248604003197068937102288102382936973e-4340 */
/*474*/ ,LDOUBLE_HEX(0791,a541,3d90,0000,0000)  /* 2.701636227045946740911872991511884329245546902342118267324793e-4349 */
/*475*/ ,LDOUBLE_HEX(0771,e7cc,6e4c,0000,0000)  /* 8.823147451116102484611454272986948405161640359102299652212965e-4359 */
/*476*/ ,LDOUBLE_HEX(0753,e80a,a8fa,0000,0000)  /* 8.225813622993712786287518061648304674330824117979650846039830e-4368 */
/*477*/ ,LDOUBLE_HEX(0734,c3a7,4e14,0000,0000)  /* 3.229763862889700675685803520695083138886794739053764677967895e-4377 */
/*478*/ ,LDOUBLE_HEX(0712,bad9,f7e0,0000,0000)  /* 1.795393567534189841173348395980344688847039679453389534601212e-4387 */
/*479*/ ,LDOUBLE_HEX(06f5,eff2,fea0,0000,0000)  /* 4.294500396525460378466276071634014814642071400631931267523176e-4396 */
/*480*/ ,LDOUBLE_HEX(06d7,bd48,cef6,0000,0000)  /* 3.155065037230420821401504224282838283405894723575669884951448e-4405 */
/*481*/ ,LDOUBLE_HEX(06b7,f559,7544,0000,0000)  /* 9.521792097115654044143909708250499402914902974903933001058218e-4415 */
/*482*/ ,LDOUBLE_HEX(0698,f655,2388,0000,0000)  /* 4.451696950568582395647008614794448597149280635851727456011937e-4424 */
/*483*/ ,LDOUBLE_HEX(0676,ad56,e900,0000,0000)  /* 1.823397760362981859127567287744136433408542992452249402385424e-4434 */
/*484*/ ,LDOUBLE_HEX(0659,8437,2cc0,0000,0000)  /* 2.590576850555743602817906884140384308039239535529686153831598e-4443 */
/*485*/ ,LDOUBLE_HEX(063b,c544,3068,0000,0000)  /* 3.599707312857097418127620380666353794557224223328546187065263e-4452 */
/*486*/ ,LDOUBLE_HEX(061b,e73c,dc70,0000,0000)  /* 9.824557611080523725019945476282483341297945832433488586539719e-4462 */
/*487*/ ,LDOUBLE_HEX(05fc,9d49,ee50,0000,0000)  /* 3.111875427314937593445501245892146557890707610390320929597326e-4471 */
/*488*/ ,LDOUBLE_HEX(05df,81f6,93d2,0000,0000)  /* 4.789331774779646229233028780726636036826226500179000498619071e-4480 */
/*489*/ ,LDOUBLE_HEX(05bf,adff,941c,0000,0000)  /* 1.492931317128242386117117361847983103158255219178686876979912e-4489 */
/*490*/ ,LDOUBLE_HEX(05a1,8e45,2ce2,0000,0000)  /* 1.136864774948321469235623773822523320321788211364230763539868e-4498 */
/*491*/ ,LDOUBLE_HEX(0581,95fd,4a60,0000,0000)  /* 2.790587036477460600732898792560497825997690813741394368983155e-4508 */
/*492*/ ,LDOUBLE_HEX(0563,b83a,79f8,0000,0000)  /* 3.192211985940410743091088478964862524646165631777546695777737e-4517 */
/*493*/ ,LDOUBLE_HEX(0543,a1f5,cb58,0000,0000)  /* 6.534063552074127995117956343206868543093545976533673202475612e-4527 */
/*494*/ ,LDOUBLE_HEX(0525,dc36,2a3a,0000,0000)  /* 8.273999939194716752137385495365199951114291619953196731532234e-4536 */
/*495*/ ,LDOUBLE_HEX(0506,890e,14ee,0000,0000)  /* 2.397951510777458191959539155218288195138014774355498820878845e-4545 */
/*496*/ ,LDOUBLE_HEX(04e3,c432,8f20,0000,0000)  /* 9.990543249724095395057752481314112689487722867971568729761937e-4556 */
/*497*/ ,LDOUBLE_HEX(04c6,b21d,8440,0000,0000)  /* 1.689378619613666111746016695401646755223736605544995850487559e-4564 */
/*498*/ ,LDOUBLE_HEX(04a7,f698,5430,0000,0000)  /* 1.089131278470334943453078672603329616241698146662428706935514e-4573 */
/*499*/ ,LDOUBLE_HEX(048a,8b20,04ae,0000,0000)  /* 1.144541196227532834417813282593965938302797357322027864411336e-4582 */
/*500*/ ,LDOUBLE_HEX(046b,e872,04d0,0000,0000)  /* 8.904654901708921244833091290514050378491284096292720610006732e-4592 */
/*501*/ ,LDOUBLE_HEX(044c,8893,c78e,0000,0000)  /* 2.436378201808789842374100076705266386330367429139215706408297e-4601 */
/*502*/ ,LDOUBLE_HEX(042b,9895,0f78,0000,0000)  /* 3.168695483203923313075059713716915930375674879982839923345483e-4611 */
/*503*/ ,LDOUBLE_HEX(040d,df2e,11d4,0000,0000)  /* 4.316500200313474395882978260786601303621825551148707531267976e-4620 */
/*504*/ ,LDOUBLE_HEX(03ef,89d5,e84c,0000,0000)  /* 2.482778785682295091270945727485215005810466431630816875264852e-4629 */
/*505*/ ,LDOUBLE_HEX(03d0,8a67,f7be,0000,0000)  /* 1.160919595871910497326959721557808670870133237957693223531261e-4638 */
/*506*/ ,LDOUBLE_HEX(03b1,c906,95d0,0000,0000)  /* 7.851788696298473970847426176741549921590771517268829264411706e-4648 */
/*507*/ ,LDOUBLE_HEX(0392,d96c,64ce,0000,0000)  /* 3.954516694686897290306947394139735293952159235706959954444691e-4657 */
/*508*/ ,LDOUBLE_HEX(0373,a16f,97ba,0000,0000)  /* 1.367279750269596271630331666607590801620953256249275824915777e-4666 */
/*509*/ ,LDOUBLE_HEX(0354,c51a,463c,0000,0000)  /* 7.773557253091627024361163166561289523837150306963002174353435e-4676 */
/*510*/ ,LDOUBLE_HEX(0335,e93e,ef0a,0000,0000)  /* 4.283624608208400742681777245592076129386385999492501824412928e-4685 */
/*511*/ ,LDOUBLE_HEX(0316,a466,526a,0000,0000)  /* 1.405946029111208854236974538415761789138817513470365204320134e-4694 */
/*512*/ ,LDOUBLE_HEX(02f5,a94d,4140,0000,0000)  /* 1.685540923164455401019567047205519247942821678840081326521648e-4704 */
/*513*/ ,LDOUBLE_HEX(02d8,ce68,b4ec,0000,0000)  /* 3.827688284800605745040265187285570356617168169635824415629816e-4713 */
/*514*/ ,LDOUBLE_HEX(02b9,8490,4fa2,0000,0000)  /* 1.144729070308467503222833635674691735871763479321236555967759e-4722 */
/*515*/ ,LDOUBLE_HEX(0298,ca98,4e60,0000,0000)  /* 2.036651845796114397993521544996177305589382294611629778831103e-4732 */
/*516*/ ,LDOUBLE_HEX(027a,e09c,08dc,0000,0000)  /* 2.102889531716642029390555194877680721796931734458725537058131e-4741 */
/*517*/ ,LDOUBLE_HEX(025c,c7e3,2b90,0000,0000)  /* 1.742905464239328306664546858116014149298371476791857344948288e-4750 */
/*518*/ ,LDOUBLE_HEX(023c,f8ef,5a7c,0000,0000)  /* 5.053758732736767439275562522146736469299997099184984300984031e-4760 */
/*519*/ ,LDOUBLE_HEX(021e,9d58,5022,0000,0000)  /* 2.974963136968326144084010884180333013989110740826827285011460e-4769 */
/*520*/ ,LDOUBLE_HEX(01ff,e242,553a,0000,0000)  /* 1.992071544035527984780359982032745912348114309137768534021568e-4778 */
/*521*/ ,LDOUBLE_HEX(01e0,b54d,7e6e,0000,0000)  /* 7.433156604186033290690902191078996301127310524735627229864350e-4788 */
/*522*/ ,LDOUBLE_HEX(01c1,a88c,d2d4,0000,0000)  /* 3.217867180381048944292126442687300327577533088876457044176624e-4797 */
/*523*/ ,LDOUBLE_HEX(01a2,fe6d,75b6,0000,0000)  /* 2.261900192516805509061632262137309798059749236804394707841782e-4806 */
/*524*/ ,LDOUBLE_HEX(0183,f04c,3bb2,0000,0000)  /* 9.947848406447613643751013148688351900759302246602784932858360e-4816 */
/*525*/ ,LDOUBLE_HEX(0163,c046,c2d0,0000,0000)  /* 1.853299177920818162186560617418402509814371295014981498183669e-4825 */
/*526*/ ,LDOUBLE_HEX(0145,f9d6,250c,0000,0000)  /* 2.242722551864474981369444061258725578342100626354047650983679e-4834 */
/*527*/ ,LDOUBLE_HEX(0126,8a0d,b7b4,0000,0000)  /* 5.770819351130121196712358466831696699832607291310186355872996e-4844 */
/*528*/ ,LDOUBLE_HEX(0107,e4f5,1a72,0000,0000)  /* 4.456713102459032263154660234244818735338244373393227640581077e-4853 */
/*529*/ ,LDOUBLE_HEX(00e8,a9b8,825c,0000,0000)  /* 1.538384784468695898088349317112475891676641057537820671019429e-4862 */
/*530*/ ,LDOUBLE_HEX(00c6,c9ad,6c90,0000,0000)  /* 1.064063445938631349967948819151338848629768493448181290220368e-4872 */
/*531*/ ,LDOUBLE_HEX(00aa,d1bb,4d00,0000,0000)  /* 4.122249658244384254548568099910954026336347617843400944272435e-4881 */
/*532*/ ,LDOUBLE_HEX(0087,ee01,a4a0,0000,0000)  /* 1.361473368623084131880201373239639182989918256198419508873590e-4891 */
/*533*/ ,LDOUBLE_HEX(006b,be9f,89a8,0000,0000)  /* 4.062153645767111065906847680169594757049848160778754438122900e-4900 */
/*534*/ ,LDOUBLE_HEX(004d,c80c,038e,0000,0000)  /* 3.970202173060005854722890052320865613293004832697137236963098e-4909 */
/*535*/ ,LDOUBLE_HEX(002e,8384,fe26,0000,0000)  /* 1.215460995941272213716595250925952040526828159082849397148943e-4918 */
/*536*/ ,LDOUBLE_HEX(000e,ca57,cd90,0000,0000)  /* 4.353907010166364145667177331653113842444894241749329113959176e-4928 */
};

 /*
 /   x - исходный аргумент
 /   lo - указатель на младшую часть возвращаемого результата;
 /   nseg - указатель на целое, содержащее 2 младших разряда
 /          целого числа, ближайшего к значению [x*pi/2].
 /
 /   Функция возвращает старшую часть приведенного аргумента.
*/

double __rem_pio2l (long double x,
                    double * lo,
                    int * nseg)
{
    int i, j , k , exp, bitpos, sign;
    long double xl, xh, y, yl, yh, z, z1, z2, t, s;
    bits_80f *ptx = (bits_80f *) &x;
    bits_80f *pty = (bits_80f *) &y;
    long double VOLATILE_80 tt;

    sign = ptx->sign;
    ptx->sign = 0;

    if (ptx->exponent < BIAS_80F+30) {   /* |x| < 2^30: use Cody & Waite algorithm */
        /* k = (int) trunc(x*(4/pi)) */
        y = x * _4onpi_d;
        k = pty->significand >> (BIAS_80F+63 - pty->exponent);
        j = k + (k & 1);
        k = (((k + 1) >> 1) * (1 - (sign << 1))) & 3;

        /* yh+yl = x - k*(Pi/4) */
        if (ptx->exponent < BIAS_80F+17) {   /* 47x3 */
            z = ( (long double)j * (long double)pi04_3d[0] );
            y = (x - z);
            z = y*TWO_32H; tt = (y + z); yh = (tt - z); yl = (y - yh);
            z = ( (long double)j * (long double)pi04_3d[1] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
            z = ( (long double)j * (long double)pi04_3d[2] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
        } else {                            /* 32x5 */
            z = ( (long double)j * (long double)pi04_5d[0] );
            y = (x - z);
            z = y * TWO_32H; tt = (y + z); yh = (tt - z); yl = (y - yh);
            z = ( (long double)j * (long double)pi04_5d[1] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
            z = ( (long double)j * (long double)pi04_5d[2] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
            z = ( (long double)j * (long double)pi04_5d[3] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
            z = ( (long double)j * (long double)pi04_5d[4] );
            t = yh; yh -= z; t -= yh; t -= z; yl += t;
        }
    } else {                            /* |x| >= 2^30: use Payne & Hanek algorithm */
        exp = ptx->exponent;
        if (exp == 32767) {
            if (x == x) /* x is inf */
	        __set_errno (EDOM);
            *nseg = 8;
            *lo = x-x;
            return x-x; /* NaN */
        }
        x *= SCALE;
        /* split x into hi+lo pair (use truncation instead of rounding!) */
        xl = x;
        ptx->significand &= 0xffffffff00000000ULL; /* strip off low 32 bits */
        xh = x; xl -= xh;
        /* find integer part of x*(4/pi) */
        bitpos = exp - (BIAS_80F+34);
        j = (bitpos / 31);
        bitpos = bitpos - j*31;
        if (bitpos >= 17) { /* original: '>= 23' */
            /* yh+yl: y = (xl*_4onpi_31l[j+0] + xh*_4onpi_31l[j+1]); j++; */
            yh = xl * _4onpi_31l[j+0]; z = xh * _4onpi_31l[j+1]; j++;
            t = yh; yh += z; t -= yh; yl = (t + z); y = (yh + yl);
            pty->significand &= 0xffffffffff000000ULL; /* strip off low 24 bits */
            yh -= y;
        } else {
            yh = yl = 0;
        }
        /* yh+yl: y += (xl*_4onpi_31l[j+0] + xh*_4onpi_31l[j+1]); */
        z = xl * _4onpi_31l[j+0]; t = yh; yh += z; t -= yh; t += z; yl += t;
        z = xh * _4onpi_31l[j+1]; t = yh; yh += z; t -= yh; t += z; yl += t;
        y = (yh + yl);
        /* y += (z1 + z2) = (xl*_4onpi_31l[j+1] + xh*_4onpi_31l[j+2]); */
        z1 = xl * _4onpi_31l[j+1]; y += z1;
        z2 = xh * _4onpi_31l[j+2]; y += z2;
        /* y = (int) trunc(y) */
        i = (63+BIAS_80F - pty->exponent);
        pty->significand = (pty->significand >> i) << i;
        k = (pty->significand >> i);

        /* yh+yl: y -= (int)y; */
        yh -= y; t = yh; yh += yl; t -= yh; yl += t;
        /* j += 2; yh+yl: y += sum(xl*_4onpi_31l[j+0] + xh*_4onpi_31l[j+1],j=0..4) */
        yh += zero_none[k & 1];
        k = (((k + 1) >> 1) * (1 - (sign << 1))) & 3;
        /* yh+yl: y += (z1 + z2); */
        t = yh; yh += z1; t -= yh; t += z1; yl += t;
        t = yh; yh += z2; t -= yh; t += z2; yl += t;
        /* yh+yl: y += (xl*_4onpi_31l[j+2] + xh*_4onpi_31l[j+3]); */
        z = xl * _4onpi_31l[j+2]; t = yh; yh += z; t -= yh; t += z; yl += t;
        z = xh * _4onpi_31l[j+3]; t = yh; yh += z; t -= yh; t += z; yl += t;
        /* yh+yl: y += (xl*_4onpi_31l[j+3] + xh*_4onpi_31l[j+4]); */
        z = xl * _4onpi_31l[j+3]; t = yh; yh += z; t -= yh; t += z; yl += t;
        z = xh * _4onpi_31l[j+4]; t = yh; yh += z; t -= yh; t += z; yl += t;
        /* yh+yl: y += (xl*_4onpi_31l[j+4] + xh*_4onpi_31l[j+5]); */
        z = (xl * _4onpi_31l[j+4] + xh * _4onpi_31l[j+5]);
        t = yh; yh += z; t -= yh; t += z; yl += t;
        /* yh+yl: y += (xl*_4onpi_31l[j+5] + xh*_4onpi_31l[j+6]); */
        z = (xl * _4onpi_31l[j+5] + xh * _4onpi_31l[j+6]);
        t = yh; yh += z; t -= yh; t += z; yl += t;
        /* yh+yl: y += (xl*_4onpi_31l[j+6] + xh*_4onpi_31l[j+7]); resplit yh+yl; */
        z = (xl * _4onpi_31l[j+6] + xh * _4onpi_31l[j+7]);
        t = yh; yh += z; s = yh * TWO_32H; tt = (yh + s);
        yh = (tt - s); t -= yh; t += z; yl += t;
        /* yh+yl: y *= Pi/4 */
        yl = pi04_2d[0]*yl + pi04_2d[1] * (yh + yl); yh = pi04_2d[0] * yh;
    }
    /* final resplitting of yh+yl and return */
    t = yh; y = (yh + yl); z = y*TWO_12H; tt = (y + z);
    yh = (tt - z); t -= yh; yl += t;
    if (sign) {
        yl = -yl;
        yh = -yh;
    }
    *nseg = k;
    *lo = yl;
    return yh;
}
